#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "ipincl.h"
#define MINFREEMEM  235000L   /* Our estimate of minimum free memory needed */
#define MINTOTALMEM 429056L   /* Our estimate of the total memory needed to run. */

FileName Config_name = "config.gcf"; /* name of configuration file */
				 /* must be in directory with intlpop.exe */

/***********************************************************************
Local reader for the country list file.

GLOBALS: Datapath, Country_list;
***********************************************************************/
boolean list_reader(
  BigString buffer,   /* string to process */
  ReaderStatusType status,          /* what to do */
  void *data
)
{
  BigString string;
  BigString cfilename;
  int country_cnt = 0; /* number of countries successfully processed */
  int region_cnt = 0;  /* number of regions successfully processed */
  int numkwds;         /* numer of keywords in format statement */
  FormatList format;   /* space for format keywords */
  int slot = 0;        /* position in country list */
  int i;


  data = data; /*--- to keep compiler happy ---*/
  if (status == READ_INIT)
  {
    Numregions = -1;  /* So we can tell if it has been processed */
    return(TRUE);
  }
  if(status == READ_FINAL)
  {
    return(TRUE);
  }

  if (strcmp(buffer, "numregions=") == 0)
    Numregions = (int)GSrdinteger();
  else if (strcmp(buffer, "numcountries=") == 0)
  {
    if (Numregions == -1) /* This must have been done already to continue */
      GSelog (-1, "Processing Numcountries but don't know Numregions\n");
    Numcountries = (int)GSrdinteger();
    if ((Country_list = (Clist *)calloc(Numregions+Numcountries,
                                        sizeof (Clist))) == NULL)
      GSelog (-1, "Out of memory for Country_list\n");
    for(i=0; i<Numregions+Numcountries; i++)
    { /* Initialize Country_list -- not all slots may be filled in later */
      Country_list[i].file[0] = '\0';
      Country_list[i].name[0] = '\0';
      Country_list[i].code = -1;
      Country_list[i].color = -1;
      Country_list[i].display = FALSE;
    }
  }
  else if (strcmp(buffer, "format=") == 0)
  { /* process format statement */
    if (!GSrdformat(&numkwds, format))
    {
      return(FALSE); /* GSrdformat should say why */
    }
    if ((numkwds != 5) ||
        (strcmp(format[0], "type") != 0) ||
        (strcmp(format[1], "filename") != 0) ||
        (strcmp(format[2], "name") != 0) ||
        (strcmp(format[3], "code") != 0) ||
        (strcmp(format[4], "color") != 0))
    {
      return (TRUE);  /* not the right format statement.  Skip this. */
    }
    /* Suck up the names.  Verify the files' existance as we go.
       If the file can't be opened, then don't include it. */
    if (!GSrdword(string, BIGSTRINGLEN))
      return(FALSE);
    while (strcmp(string, "endformat") != 0)
    {
      if ((strcmp(string, "sub") != 0) &&
          (strcmp(string, "region") != 0) &&
          (strcmp(string, "none") != 0))
      {
        return(FALSE);  /* something wrong with this format */
      }
      if(strcmp(string, "none") == 0)
      { /* this is a country */
        if(country_cnt >= Numcountries)
        {
          GSelog (-1, "list_reader: Too many countries!\n");
        }
        country_cnt++;
      }
      else
      { /* region or subregion */
        if (region_cnt >= Numregions)
        {
           GSelog(0, "list_reader: Too many countries!\n");
           return(FALSE);
        }
        region_cnt++;
      }
      if (!GSrdword(Country_list[slot].file, FILENAMELEN))
      {
        return(FALSE);
      }
      if (!GSrdphrase(Country_list[slot].name, NAMELENGTH))
      {
        return(FALSE);
      }

      /* Check for duplicate names */
      if (find(Country_list, Country_list[slot].name, 0, slot) != -1)
      {
        Country_list[slot].display = FALSE;
      }
      else
      {
        Country_list[slot].display = TRUE;
      }

      Country_list[slot].code = (int)GSrdinteger();
      if (!GSrdword(string, FILENAMELEN))
      {
        return(FALSE);
      }
      Country_list[slot].color = GSconvertcolor(string);
      sprintf(cfilename, "%s%s", Datapath, Country_list[slot].file);
      if (!GSrdword(string, BIGSTRINGLEN))
      {
        return(FALSE);
      }
      slot++;

    } /* while */

    Realnumregions = region_cnt;
    Realnumcountries = country_cnt;
  } /* if format= */
  return(TRUE);
}


/***********************************************************************
Initiate reader for country database list file.

GLOBALS: Listname, Country_list.
***********************************************************************/
void get_country_names(void)
{
  FILE *list_fp;

  if ((list_fp = GSopen(ListFile, "rt")) == NULL)
    GSelog(-1, "Unable to access country list |%s|.\n", ListFile);
  if (!GSgenreader(list_fp, list_reader, "IP_clist", NULL))
    GSelog(-1, "Error processing country database list file |%s|\n", ListFile);
  fclose(list_fp);
}



/***********************************************************************
Local reader for the configuration file.

GLOBALS: Listname, Datapath, Infopath (configuration variables).
***********************************************************************/
boolean config_reader(
  BigString buffer, /* string to process */
  ReaderStatusType status,        /* what to do */
  void *data
)
{
  BigString tmpstr;

  data = data; /*--- to keep compiler happy ---*/
  if(status == READ_INIT)
    return(TRUE);  /* nothing to do */
  if(status == READ_FINAL)
    return(TRUE);  /* nothing to do */

  if (GSstricmp(buffer, "listname=") == 0)
  {
    if(!GSrdword(tmpstr, BIGSTRINGLEN))
      return(FALSE);
    sprintf(Listname, "%s%s", Home, tmpstr);
  }
  else if (GSstricmp(buffer, "datapath=") == 0)
  {
    if(!GSrdword(tmpstr, BIGSTRINGLEN))
      return(FALSE);
    sprintf(Datapath, "%s%s", Home, tmpstr);
  }
  else if (GSstricmp(buffer, "listpath=") == 0)
  {
    if(!GSrdword(tmpstr, BIGSTRINGLEN))
      return(FALSE);
    sprintf(Listpath, "%s%s", Home, tmpstr);
  }
  else if (GSstricmp(buffer, "infopath=") == 0)
  {
    if(!GSrdword(tmpstr, BIGSTRINGLEN))
      return(FALSE);
    sprintf(Infopath, "%s%s", Home, tmpstr);
  }
  else if (GSstricmp(buffer, "imagepath=") == 0)
  {
    if(!GSrdword(tmpstr, BIGSTRINGLEN))
      return(FALSE);
    sprintf(Imagepath, "%s%s", Home, tmpstr);
  }
  else if (GSstricmp(buffer, "interfacefile=") == 0)
  {
    if(!GSrdword(tmpstr, BIGSTRINGLEN))
      return(FALSE);
    sprintf(InterfaceFile, "%s%s", Home, tmpstr);
  }
  else if (GSstricmp(buffer, "instructionfile=") == 0)
  {
    if(!GSrdword(tmpstr, BIGSTRINGLEN))
      return(FALSE);
    sprintf(Introfilename, "%s%s", Home, tmpstr);
  }
  else if (GSstricmp(buffer, "countryfile=") == 0)
  {
    if(!GSrdword(tmpstr, BIGSTRINGLEN))
      return(FALSE);
    sprintf(Countryfilename, "%s%s", Home, tmpstr);
  }
  else if (GSstricmp(buffer, "regionfile=") == 0)
  {
    if(!GSrdword(tmpstr, BIGSTRINGLEN))
      return(FALSE);
    sprintf(Regionfilename, "%s%s", Home, tmpstr);
  }
  else if (GSstricmp(buffer, "bstore=") == 0)
    Use_bstore = GSrdboolean();
  else if (GSstricmp(buffer, "fpu=") == 0)
    HasFPU = GSrdboolean();
  else if (GSstricmp(buffer, "graphfileformat=") == 0)
  {
    if(!GSrdword(tmpstr, BIGSTRINGLEN))
    {
      return(FALSE);
    }
    if (GSstricmp(tmpstr, "BMP16") == 0)
    {
      GraphFileFmt = BMP_16;
    }
    else if (GSstricmp(tmpstr, "PSGREY") == 0)
    {
      GraphFileFmt = PS_GRAY;
    }
    else if (GSstricmp(tmpstr, "PSCOLOR") == 0)
    {
      GraphFileFmt = PS_COLOR;
    }
    else if (GSstricmp(tmpstr, "GIF16") == 0)
    {
      GraphFileFmt = GIF_16;
    }
    else
    {
      GraphFileFmt = NO_FILE;
    }
  }
  return(TRUE);
}


/***********************************************************************
Process the configuration file -- initiate reader.

STATIC GLOBALS: Config_name.
***********************************************************************/
void configure(void)
{
  FILE *config_fp;
  BigString filename;

  sprintf(filename, "%s%s", Home, Config_name);
  if ((config_fp = GSopen(filename, "rt")) == NULL)
    return; /* Unable to open the configure file - use defaults. */
  if (!GSgenreader(config_fp, config_reader, "IP_config", NULL))
    GSelog(-1, "Error while reading configuration file |%s|\n", filename);
  fclose(config_fp);
}


/**********************************************************************
Print out the usage message and exit.
GLOBALS USED: None.
**********************************************************************/
void printusage(void)
{
  fprintf(stderr, "International Population.\n");
  fprintf(stderr, "\tUsage: Intlpop [-d] [-t] [-w] [-l <savefile>] [-cmap x y name]\n");
  fprintf(stderr, "The -d option turns on debug messages\n");
  fprintf(stderr, "The -t option enables 'Glossary' button\n");
  fprintf(stderr, "The -w option starts user up in Walk Through tutorial\n");
  GSdelay(ERRORDELAY);
  exit(0);
}

/***********************************************************************
Main routine: Do initializations and start interface interpreter.

GLOBALS: None.
***********************************************************************/
int GS_MAIN(GS_MAIN_ARGS_F)
{
  int i, j;
  boolean loadfile = FALSE;  /* If true, then load a simulation file */
  BigString *getname = NULL;    /* If not null, then get user's name at log init */
  boolean cmapflag = FALSE;  /* If true, run cmap functions */
  char *init_file_mode = "w";/* mode string for opening output log file */
  long filesize;
  FILE *fp; /* For maps */
  void *mem;
  GSColor color;
  int lwd, lht;
  int x, y, wd, ht;

  /* Get the path to the directory containing the executable */
 /* strncpy(Home, argv[0], BIGSTRINGLEN);*/
  Home[0] = '\0';	
  if (strlen(Home) >= BIGSTRINGLEN)
  {
    printf("Weird problem with |%s|\n", Home);
    GSdelay(ERRORDELAY);
    exit(-1);
  }
  for(i=strlen(Home); (i>=0) && (Home[i]!='\\') && (Home[i]!='/'); i--)
    Home[i] = '\0';
  i = 1;
 #if 0
  while (i < argc)
  {
    if (GSstricmp (argv[1], "-d") == 0)
      GSDebug = TRUE;
    else
    if (GSstricmp (argv[i], "-w") == 0)
      StartWalkThru = TRUE;
    else
    if (GSstricmp (argv[i], "-l") == 0)
    {
      i++;
      if (i < argc)
	strcpy (SaveFile, argv[i]);
      loadfile = TRUE;
    }
    else
    if (strcmp (argv[i], "-t") == 0)
    {
      TutorEntry = TRUE;
      /* Test to see if TUTOR_SAVE exists */
	  if((tutor_fp = GSopen(TUTOR_SAVE, "r")) != NULL)
      {
 	    if(fclose(tutor_fp) != 0)
	    {
           GSbell();
	       GSelog(0, "Funny problem - couldn't close |%s|\n", TUTOR_SAVE);
	    }
		strcpy (SaveFile, TUTOR_SAVE);
	    Tutorfile = TRUE;
        strcpy(init_file_mode, "a");
        if (getname != NULL)
          free (getname);
        getname = NULL;
      }
      else
	    Tutorfile = FALSE;
    }
    else
    if (GSstricmp (argv[i], "+n") == 0)
      getname = (BigString *)malloc(sizeof(BigString));
    else
    if (GSstricmp (argv[i], "-cmap") == 0)
    {
      cmapflag = TRUE;
      i++; x = atoi (argv[i]);
      i++; y = atoi (argv[i]);
    }
    else
      printusage ();
	i++;
  }
#endif
  if (GSmemavail() < MINFREEMEM)
    GSelog (-1, "You must have at least %ld bytes (%ld Kilobytes) of free memory \nto run this program.\n",
                MINTOTALMEM, (MINTOTALMEM / 1024));
  sprintf(Introfilename, "%s%s", Home, IDEFAULT);
  sprintf(Countryfilename, "%s%s", Home, CDEFAULT);
  sprintf(Regionfilename, "%s%s", Home, RDEFAULT);

  HasFPU = GShasFPU();
  configure();
  sprintf(VersionFile, "%s%s", Infopath, POP_HLP);
  if (init_sim_arrays () == 0)
    GSelog (-1, "Out of memory for simulation arrays.\n");

  sprintf(CountryFile, "%s%s", Imagepath, Countryfilename);
  sprintf(RegionFile, "%s%s", Imagepath, Regionfilename);
  sprintf(IntroFile, "%s%s", Imagepath, Introfilename);
  sprintf(ListFile, "%s%s", Listpath, Listname);
  if ((fp = GSopen(CountryFile, "rb")) == NULL)
    GSelog(-1, "Unable to open country map |%s|\n", CountryFile);
  filesize = GSfilesize(fp);
  if ((mem = (void *)malloc(filesize-8)) == NULL)
    GSelog(-1, "Unable to allocate space for country map\n");
  if (!GSinitpic(fp, &Countrypic, mem))
    GSelog(-1, "Unable to initialize country map\n");
  fclose(fp);
  if ((fp = GSopen(RegionFile, "rb")) == NULL)
    GSelog(-1, "Unable to open region map |%s|\n", RegionFile);
  filesize = GSfilesize(fp);
  if ((mem = (void *)malloc(filesize-8)) == NULL)
    GSelog(-1, "Unable to allocate space for region map\n");
  if (!GSinitpic(fp, &Regionpic, mem))
    GSelog(-1, "Unable to initialize region map\n");
  fclose(fp);

  GSinterfaceinit("International Population", InterfaceFile, Use_bstore,
                  init_file_mode, G640x480x16, GS_MAIN_ARGS_A);

  if (!GSinitlog (init_file_mode))
  {
    printf("Unable to initialize output.log\n");
    GSdelay(ERRORDELAY);
    exit(-1);
  }

  GSgetfieldrect("LMAP", &x, &y, &lwd, &lht, &color);
  GSgetfieldrect("SMAP", &x, &y, &wd, &ht, &color);
  compute_xorbox(wd/2, ht/2, MapScale, wd, ht, lwd, lht);
  get_country_names();
#if 0
  if (cmapflag)
  {
	makecmap (argv[i], x, y);
	GSelog (-1, "Color map |%s| reprocessed.\n", argv[i]);
  }
#endif
 for (i=0; i<NUM_DISPLAYS; i++)
	Country[i].country_name[0] = '\0';
  for (i=0; i<NUM_DISPLAYS; i++)
    for (j=0; j<NUM_SIMS; j++)
      if ((IPState[i][j] = (IPStatetype *)malloc(sizeof (IPStatetype))) == NULL)
	GSelog(-1, "Unable to allocate space for simulation %d, %d\n", i, j);
      else
	IPState[i][j]->stepcnt = -1;

  for (i=0; i<NUM_DISPLAYS; i++)
    for (j=0; j<NUM_SIMS; j++)
      if ((IPsaveyear[i][j] = (IPsaveyear_t *)malloc(sizeof(IPsaveyear_t)))
           == NULL)
	GSelog(-1, "Unable to allocate space for simulation %d, %d\n", i, j);

  if (TutorEntry)
  {
    GSdisablebutton("root", "File", NO_DRAW);
    GSinvisiblebutton("root", "File", NO_DRAW);
  }
  else
    GSinvisiblebutton("root", "Glossary", NO_DRAW);

  if (loadfile || Tutorfile)
  {
    GSsetdeactivenamedwindow ("instruction");
    GSsetactivenamedwindow ("top");
    GSsetactivenamedwindow ("bottom");
    GSactivatefunction ("load_simulation");
  }
  else
    GSactivatefunction ("do_intro");

  /*GSelog(0, "Home directory is |%s|.\n", Home);*/
  GSelog(0, "Memory Available: %ld.\n", GSmemavail());
  /*GSelog (0, "Version File is |%s|\n", VersionFile);*/

  if (HasFPU)
    GSelog(0, "Floating point will be used for calculations.\n");
  else
    GSelog(0, "Floating point will not be used for calculations.\n");


  GSinterface();

  return(0);
}
