/***********************************************************************
International Population module include file.
***********************************************************************/
#include "geosim.h" /* Needed for "GSPicture" type */

/* Help text files.  These should be found in directory InfoPath */
#define INTRO_HLP    "tutor1.hlp"
#define POP_HLP      "pop.txt"
#define GEO_HLP      "geo.txt"
#define TECH_HLP     "tech.txt"
#define MAIN_HLP     "main.hlp"
#define OPTIONS_HLP  "options.hlp"
#define CONTROLS_HLP "controls.hlp"
#define EXIT_HLP     "modeling.txt"
#define TUTOR_SAVE   "ip_tut.sav"
#define LIST_FILE    "country.gaf"

#define NUM_DISPLAYS 2  /* number of country displays */
#define NUM_POINTS  13  /* number of simulation plot points (1990-2050) */
#define NUM_SIMS     3  /* number of simulations in country display */
#define DISPLAY1     0  /* upper country display */
#define DISPLAY2     1  /* lower country display */
#define TIME_STEP    5  /* Number of cycles (years) in a time step */
#define FEMALEPERCENT 512 /* Percentage of female babies per 1000 (105:100) */
#define NAMELENGTH  21  /* Maximum length for a country name */
#define MAXRATE  400000L/* Maximum allowed birth/death rate */
#define COHORTS 85      /*  one per year of data */
#define RATERES 1000L   /* Rescale birth/death rates to parts per million */
#define LETFRRES 100.0  /* Rescale life expectancy and TRF to 100ths of years */
#define MAXTFR   990    /* Max allowed TFR 9.9 * LETFRRES */
#define PICKSPERSCROLLPAGE  23
#define MAX_OUT_SEARCH_LN 14

#define IDEFAULT "intro.rl4" /* Quick start instruction name default */
#define RDEFAULT "regions.rl8" /* Regions map name default */
#define CDEFAULT "regions.rl8" /* Countrys map name default */

typedef char Name[NAMELENGTH+1];  /* A country name string */

typedef struct
{ /* Descriptor for country display state */
  boolean modify_flag;
  boolean reset_permit;
  boolean modify_permit;
  boolean scenario_permit;
  boolean alt_permit;
  int curr_sim;
  int bound;         /* Percent bound for pyramid graph (1 = 1%) */
  long maxpop;     /* Greatest population seen for this display */
  long intuppop;   /* Current upper bound for the population graph */
  long male[17];   /* Initial bar populations for the current step of the */
  long female[17]; /*   current simulation - used by the drag bar routine */
} DisplayType;

typedef struct
{ /* This structure contains all of the information initially describing
     a country. */
  Name country_name;  /* Name of country */
  char title[40];     /* Database-supplied title for country */
  int scale_factor;   /* Interpret numbers in 1's or 1000's */
  int life_adjust;    /* Adjustment factor for difference between
                         male and female life expectancies. x 1000 */
  int life_expectancy; /* in 100ths of years */
  int tot_fertility_rate; /* TFR in 100ths of children */

  /* Information for year 2000 to use in "expected scenario" */
  int life_expect_2000;   /* in 100ths of years */
  int tot_fertility_2000; /* in 100ths of children */
  long infant_deaths_2000; /* per 1,000,000 */

  /* the "base" year is 1990.  Perhaps this isn't a good idea. */
  /* While the simulation uses 10 year cohorts, we process them by 5 year
     cohorts internally. */
  /* cohort totals: total,   0-4,   5-9, 10-14, 15-19, 20-24, 25-29, 30-34,
                    35-39, 40-44, 45-49, 50-54, 55-59, 60-64, 65-69, 70-74,
                    75-79, 80+ */
  long male_1990_pop[18];
  long fem_1990_pop[18];

  /* cohort births/1,000,000 females: -15, 15-19, 20-24, 25-29, 30-34, 35-39,
                                    40-44, 45-49, 50+ */
  long tot_birthrate[9];

  /* cohort deaths/1,000,000 people:  -1,   1-4,   5-9, 10-14, 15-19, 20-24,
                                   25-29, 30-34, 35-39, 40-44, 45-49, 50-54,
                                   55-59, 60-64, 65-69, 70-74, 75-79, 80+ */
  /* deaths[0] is the infant mortality rate */
  long tot_deathrate[18];
  long mig; /* number of net migrants */
} CountryInfo;


typedef struct
{ /* Store information while reading a country data-file. */
  /* Local reader will then convert to a CountryInfo structure. */

  Name country_name;      /* Name of country */
  int buddy;   /* index into "buddy" tables for filling in missing data */
  int scale_factor;       /* interpret numbers in 1's or 1000's */
  long total_pop_1990;    /* Need this since actual cohort values may be for
                             another year, and so need to be scaled to 1990 */
  int life_expect_1990;   /* 1/10'ths of years */
  int infant_deaths_1990; /* per 10,000 */
  int tot_fertility_1990; /* 1/10'ths of children */

  int life_expect_2000;   /* 1/10'ths of years */
  int infant_deaths_2000; /* per 10,000 */
  int tot_fertility_2000; /* 1/10'ths of children */

  /* cohort totals: total,   0-4,   5-9, 10-14, 15-19, 20-24, 25-29, 30-34,
                    35-39, 40-44, 45-49, 50-54, 55-59, 60-64, 65-69, 70-74,
                    75-79, 80+ */
  long tot_pop[18];
  long male_pop[18];
  long fem_pop[18];

  /* cohort births: total, -15, 15-19, 20-24, 25-29, 30-34, 35-39, 40-44,
                    45-49, 50+ */
  long tot_births[10];
  long male_births[10];
  long fem_births[10];

  /* cohort deaths: total,    -1,   1-4,   5-9, 10-14, 15-19, 20-24,
                           25-29, 30-34, 35-39, 40-44, 45-49, 50-54,
                           55-59, 60-64, 65-69, 70-74, 75-79, 80+ */
  /* deaths[1] is infant deaths */
  long tot_deaths[19];
  long male_deaths[19];
  long fem_deaths[19];
  long mig;  /* net migration */
} ReadinInfo;

#define MAXREGIONS 25
#define MAXCPAGES  3
#define PICKSPERPAGE  80
/* This structure holds everything necessary for the country name list. */
typedef struct
{
  Name name;         /* region/country's name */
  FileName file;     /* corresponding file name */
  int code;           /* logical code used in map file */
  char color;         /* color used in .clr map file */
  boolean display; /* TRUE iff item should be displayed in list */
} Clist;

typedef struct {
  int  tfr_targ_year[NUM_POINTS];     /* temporary; for setting target */
  int  lifex_targ_year[NUM_POINTS];   /* temporary; for setting target */
  int  infmort_targ_year[NUM_POINTS]; /* temporary; for setting target */
  int  mig_targ_year[NUM_POINTS];     /* temporary; for setting target */
} IPsaveyear_t;

typedef struct {
  int  stepcnt;                           /* # of simulation steps computed */
  int  year;                              /* Current year of the simulation */
  long totpop[NUM_POINTS];       /* Total pop at start of year (for plotting) */
  int  tfr_targ_val[NUM_POINTS];          /* TFR target value */
  int  tfr_targ_year[NUM_POINTS];         /* TFR target year */
  int  lifex_targ_val[NUM_POINTS];        /* Life expectancy target value */
  int  lifex_targ_year[NUM_POINTS];       /* Life expectancy target year */
  long infmort_targ_val[NUM_POINTS];      /* infant mortality target val */
  int  infmort_targ_year[NUM_POINTS];     /* infant mortality target year */
  long mig_targ_val[NUM_POINTS];          /* migration target val */
  int  mig_targ_year[NUM_POINTS];         /* migration target year */
  /*-------------------------------------------------------------------
    cohort population labels:
          0-4,   5-9, 10-14, 15-19, 20-24, 25-29, 30-34, 35-39, 40-44,
        45-49, 50-54, 55-59, 60-64, 65-69, 70-74, 75-79, 80+
  -------------------------------------------------------------------*/
  long male_pop[NUM_POINTS][17];      /* cohort male populations */
  long fem_pop[NUM_POINTS][17];       /* cohort female populations */
  /*-------------------------------------------------------------------
    cohort birthrate labels:   -15, 15-19, 20-24, 25-29, 30-34, 35-39,
			      40-44, 45-49, 50+ 
  -------------------------------------------------------------------*/
  long birthrate[NUM_POINTS][9];      /* birthrate per 10000 women */
  /*-------------------------------------------------------------------
    cohort deathrate labels:
	-1,   1-4,   5-9, 10-14, 15-19, 20-24, 25-29, 30-34, 35-39,
	 40-44, 45-49, 50-54, 55-59, 60-64, 65-69, 70-74, 75-79, 80+
  -------------------------------------------------------------------*/
  long deathrate[NUM_POINTS][18];
  long mig[NUM_POINTS];               /* net migration */
} IPStatetype;

/*typedef enum scrollbuttonstatus {ENABLED, DISABLED} Scrollbuttonstatus;*/

/*  intlpop.c  */
void get_country_names (void);
boolean config_reader (BigString, ReaderStatusType, void *);
void configure (void);
void printusage (void);

/* ipdecl.c */
extern int CurrFormatItem;
extern int Savemsg;         /* Save help message context */
extern int ParmSavemsg;
extern int QuickStartSavemsg;
extern int MapSelectSavemsg;
extern boolean Use_bstore;
extern boolean TutorEntry;
extern boolean Tutorfile;
extern boolean Drag_Used;	/* Set by dragrect functs. Checked by walkthru */
extern boolean MapSelect;   /* TRUE iff selection window is active */
extern boolean DumpResultsActive;
extern boolean DumpSuccess;
extern boolean DumpWinActive;
extern boolean QuitWinActive;
extern boolean HelpWinActive;      /* TRUE iff help window is active */
extern boolean QuickStartActive;   /* TRUE iff QuickStart Screen is active */
extern boolean InWalkThru;
extern boolean StartWalkThru;
extern boolean SetFertActive;
extern boolean SetLifeExActive;
extern boolean SetMigActive;
extern BigString DumpFileName;
extern BigString Listname;
extern BigString Imagepath;    /* path to picture files */
extern BigString Datapath;
extern BigString Infopath;
extern BigString Listpath;
extern BigString Home;
extern BigString InterfaceFile;
extern BigString VersionFile;
extern BigString ListFile; /* country name list file (with path) */
extern BigString CountryFile; /* Country map file (with full path name) */
extern BigString RegionFile; /* Region map file (with path) */
extern BigString IntroFile; /* Quick start instruction pict file (with path) */
extern BigString SaveFile;
extern BigString Introfilename;
extern BigString Countryfilename; /* Country map file */
extern BigString Regionfilename; /* Region map file */
extern GSPicture Countrypic;
extern GSPicture Regionpic;
extern int MapScale;
extern int Xorx, Xory, Xorwd, Xorht;
extern char *Displayname[NUM_DISPLAYS];
extern char *TFRname[NUM_DISPLAYS];
extern char *Lifexname[NUM_DISPLAYS];
extern char *Migname[NUM_DISPLAYS];
extern char *Infowinname[NUM_DISPLAYS];
extern char *Birthwinname[NUM_DISPLAYS];
extern char *Deathwinname[NUM_DISPLAYS];
extern char *Infantwinname[NUM_DISPLAYS];
extern IPStatetype *IPState[][NUM_SIMS];
extern IPsaveyear_t *IPsaveyear[][NUM_SIMS];
extern Clist *Country_list;
extern int Numregions;
extern int Numcountries;
extern int Realnumregions;
extern int Realnumcountries;
extern int Picklistpage;
extern int Cdisp;
extern DisplayType Display[];
extern CountryInfo Country[];
extern int Mmig[17];
extern int Fmig[17];
extern int Pickno;
extern BigString searchstring;
extern int ScrollPicknumpages;
extern boolean HasFPU;
extern BigString Helpfile; /* Space to hold the path and name of a help file */
extern GraphicsFileType GraphFileFmt;
extern boolean  WarnedNotToCoverWin;
extern int AlertNumber;


/* ipread.c */
boolean IPreader(BigString, ReaderStatusType, void *);
boolean DBconvert(ReadinInfo *, CountryInfo *);
boolean buddyOK(ReadinInfo *);

/* ipmisc.c */
long RESCALE(long, long, long);
void dointroscreen(void);
unsigned char code2color(unsigned short, int, int, int);
void compute_xorbox(int, int, int, int, int, int, int);
void writesimstats (IPStatetype *);
int checkwindow(BigString);
void setup_bargraph(void);
void display_pop_graph(int);
void display_bar_graph(int);
void display_version (void);
long rebuild_birthrate(int, IPStatetype *, long *, int);
long rebuild_deathrate(int, IPStatetype *, long *, long *, int);
void redraw_all(void);
void update_display(int);
void redraw_display(int);
void init_display_data(int);
void init_sim_data(int, int);
void drawpickdisplay(void);
void drawpickcountry(int, int, int);
boolean readcountry(int);
boolean decmapscale(void);
boolean incmapscale(void);
boolean changefert(int);
boolean changelifex(int);
boolean changemig(int);
boolean changeyear(int, IPStatetype *, int *, char *);
boolean startsim(int);
void reset_screen(boolean);
boolean changemapscale(int);
int find (Clist *, char *, int , int );
boolean ProcessSearchtext(char);
boolean handlesearch(int, int, char);
boolean loadnamedfile (char *);
boolean savenamedfile (char *);
void display_searchstring(void);
void drawintroscreen(void);
void display_fert_win(boolean);
void display_lifeex_win(boolean);
void display_mig_win(boolean);

/* sim.c */
void setup_pop(IPStatetype *, long *, long *);
void setup_drate(IPStatetype *, long *, long *, long *);
void setup_brate(IPStatetype *, long *, long *);
void reconstruct_pop(IPStatetype *, long *, long *);
int get_TFR(long *);
int get_life_expect(long *);
void set_to_tfr_target(int, long *);
void set_to_life_target(int, long *, boolean);
void calc_sim_tfr_variant(IPStatetype *);
void calc_sim_life_variant(IPStatetype *);
void calc_sim_mig_variant(IPStatetype *);
void dosim(IPStatetype *);
boolean init_sim_arrays (void);

/*  makecmap  */
void makecmap (char * , int, int);

/*  scroll.c  */
boolean scrollselectcountry(void);
/*int scrollpicksize_func(int);*/
void adjust_for_scroll(int);
char *scrollpicklabel_func(int);
void scrollpicktext_func(GSlistHandle, int);
void scrollpickin_func(int);
void scrollpickout_func(void);
void scrollpickclick_func(int);
boolean scrolldraggetcountry(int, int, DragStatusType);
boolean scrollpickcancel(void);
boolean scrolldnline(void);
boolean scrollupline(void);
/*boolean scrollup(int);
boolean scrolldn(int);*/
boolean scrollbar(int, int, DragStatusType);
/*boolean drawscroll( int, int, int, int);*/
/*void drawuparrow(ButtonStatusType, boolean);
void drawdownarrow(ButtonStatusType, boolean);*/
void draw_selection_window(void);
void endselectcountry();

/* walkthru.c */
boolean walkthruinit(void);
boolean walkthru( int, int, DragStatusType);
boolean quitwalkthru(void);

/* iroutine.c */
boolean selectcountrymap(void);
boolean mapcancel(void);
void highlightfield(BigString, BigString, GSColor, GSColor, GSColor);