/**********************************************************************
All the International Population module routines that don't belong
anywhere else (i.e., not simulation routines nor routines called by the
interface handler).
**********************************************************************/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#ifndef __MWERKS__/* values.h doesn't exist on mac */
#include <values.h>
#endif
#include "ipincl.h"

#define NUM_INTRO_COLS 16

static char *Clabel[] = {"0-9", "10-19", "20-29", "30-39", "40-49",
			"50-59", "60-69", "70-79", "80+"};

static char IntroCol[NUM_INTRO_COLS];

/***********************************************************************
A safe rescaling function for longs.  Intention is to avoid overflow
of numbers that are potentially too big.
Change to double-long word multiply -- CAS
***********************************************************************/
#define SHORTMASK 0xffff
#define TOPMASK 0x7fffffffL   /* All but topmost bit */

long RESCALE(long m1, long m2, long d)
{
  static int  loshort = -1, hishort;
  unsigned short *x, *y, *z;
  unsigned long temp, tempout[4], out[2];
  long result = 0, sign = 1;
  int i;

  if (loshort < 0)
    hishort = !(loshort = GSbigendian());

  if (d == 0)
    return(0);
  if (HasFPU) /* if FPU is present, use it and return */
    return((long)((double)m1 * (double)m2 / (double)d + 0.5));

  /* if FPU is not present, do the following: */
  if (m1 < 0) { sign *= -1; m1 *= -1; }
  if (m2 < 0) { sign *= -1; m2 *= -1; }
  if (d < 0) { sign *= -1; d *= -1; }
  x = (unsigned short *) &m1;  y = (unsigned short *) &m2;
  z = (unsigned short *) out;
  temp = (long)x[loshort] * (long)y[loshort];
  tempout[1] = temp >> 16;   z[loshort] = temp & SHORTMASK;
  temp = (long)x[loshort] * (long)y[hishort];
  tempout[2] = temp >> 16;   tempout[1] += temp & SHORTMASK;
  temp = (long)x[hishort] * (long)y[loshort];
  tempout[2] += temp >> 16;  tempout[1] += temp & SHORTMASK;
  temp = (long)x[hishort] * (long)y[hishort];
  tempout[3] = temp >> 16;   tempout[2] += temp & SHORTMASK;
  z[hishort] = tempout[1] & SHORTMASK;
  tempout[2] += tempout[1] >> 16;
  z[loshort+2] = tempout[2] & SHORTMASK;
  tempout[3] += tempout[2] >> 16;
  z[hishort+2] = tempout[3];

  if (out[1] > d)
    GSelog(-1, "RESCALE of numbers too large! %ld, %ld, %ld\n, out: %ld %ld",
           m1, m2, d, out[0], out[1]);
  if ((out[1] == 0) && (out[0] < 0x80000000L))
    result = IDIV(out[0], d);
  else
  {
    for (i=0; i<32; i++)
    {
      out[1] = (out[1] << 1) + ((out[0] & TOPMASK) != out[0]);
      out[0] <<= 1;
      result <<= 1;
      if (d < out[1])
      {
        result++;
        out[1] -= d;
      }
    } /* for */
    if ((out[1] << 1) >= d)
      result++;
  }
  result *= sign;
  return(result);
}


/***********************************************************************
Given a country list index, return the color index.
***********************************************************************/
unsigned char code2color(unsigned short code, int u1, int u2, int u3)
{
  if (code == 255)
    return((unsigned char)0); /* was -1 - JBR */
  else if (code < Realnumcountries+Realnumregions)
    return((unsigned char)Country_list[code].color);
  else
    return(GSconvertcolor("white"));
}


/***********************************************************************
Given a country list index, return the color index.
***********************************************************************/
void compute_xorbox(int cx, int cy, int scale,  /* center and scale */
                    int wd, int ht, /* small map window size */
                    int lwd, int lht) /* large map window size */
{

  Xorwd = lwd/(2*scale);
  Xorht = lht/(2*scale);
  /* cx, cy should be center of the box, but move it to fit in screen 
     if necessary */
  Xorx = cx - Xorwd/2;
  if (Xorx < 0) Xorx = 0;
  if ((Xorx + Xorwd) > wd)
    Xorx = wd - Xorwd;
  Xory = cy - Xorht/2;
  if (Xory < 0) Xory = 0;
  if ((Xory + Xorht) > ht)
    Xory = ht - Xorht;
}

unsigned char intro_color(unsigned short c)
/*char intro_color(int c)*/
{
	if ((c >= 0) && (c < NUM_INTRO_COLS))
		return IntroCol[c];
	else
		return IntroCol[0];
}

void setupintrocols(char IntroCol[])
{
	IntroCol[0] = (char) GSconvertcolor("black");
	IntroCol[1] = (char) GSconvertcolor("dgray");
	IntroCol[2] = (char) GSconvertcolor("mgray");
	IntroCol[3] = (char) GSconvertcolor("lgray");
	IntroCol[4] = (char) GSconvertcolor("bluegray");
	IntroCol[5] = (char) GSconvertcolor("lblue");
	IntroCol[6] = (char) GSconvertcolor("peach");
	IntroCol[7] = (char) GSconvertcolor("blue");
	IntroCol[8] = (char) GSconvertcolor("green");
	IntroCol[9] = (char) GSconvertcolor("lpurple");
	IntroCol[10] = (char) GSconvertcolor("red");
	IntroCol[11] = (char) GSconvertcolor("yellow");
	IntroCol[12] = (char) GSconvertcolor("lcyan");
	IntroCol[13] = (char) GSconvertcolor("magenta");
	IntroCol[14] = (char) GSconvertcolor("sandy");
	IntroCol[15] = (char) GSconvertcolor("white");
}

/***********************************************************************
  Put up the quick start insruction picture.
***********************************************************************/
void drawintroscreen(void)
{
  FILE *picfp;
  int offx, offy, wx, wy, wwd, wht;
  GSColor wcolor;

  QuickStartActive = TRUE;

  if (!InWalkThru)
    if (TutorEntry)
      GSdisablebutton("root", "glossary", DRAW);
    else
      GSdisablebutton("root", "file", DRAW);

  GSdisablemenuitem("help", "p0");
  GSdisablemenuitem("help", "p4");
  GSsetcurrentnamedwindow("instruction");
  if ((picfp = GSopen(IntroFile, "rb")) == NULL)
    GSelog(-1, "Unable to open instruction picture file |%s|\n", IntroFile);
  GSgetfieldrect ("introoffset", &offx, &offy, &wwd, &wht, &wcolor);
  GSgetfieldrect ("introfield", &wx, &wy, &wwd, &wht, &wcolor);
/*  GSwdrawfilepict(picfp, wx, wy, wwd, wht, offx, offy, NULL); */
/* This has to be modified because it depends on color order in the palette.  JBR */
  setupintrocols(IntroCol);
  GSwdrawfilepict(picfp, wx, wy, wwd, wht, offx, offy, intro_color);
/*  GSwdrawfilepict(picfp, wx, wy, wwd, wht, offx, offy, 
  	(unsigned char (*)(unsigned short)) intro_color);*/
  fclose(picfp);
  if (InWalkThru)
    GSinvisiblebutton("root", "qtwalk", DRAW);
}


/***********************************************************************
  Setup window for quickstart screen
  Put up the quick start insruction picture.
***********************************************************************/
void dointroscreen(void)
{
  GSdrawnamedwindow("instruction", NO_BSTORE, POPUP);
  drawintroscreen();
  GSdisplayhelpmessage(13);
  GSenablebutton("message", "done", NO_DRAW);
  GSsetactivenamedwindow("message");
  GSsetactivenamedwindow("root");
}



/***********************************************************************
Write out a line into the output log file when sim or unsim button
is pushed.
***********************************************************************/
void writesimstats (IPStatetype *sptr)
 {
 int stepcnt = sptr->stepcnt;

 GSolog ("%-15s: %4d %14.0f %3.1f  %4.1f %10.0f %3ld\n",
	   Country[Cdisp].country_name,
	   sptr->year,
	   (double)sptr->totpop[stepcnt]*(double)Country[Cdisp].scale_factor,
	   (double)get_TFR(sptr->birthrate[stepcnt])/LETFRRES,
	   (double)get_life_expect(sptr->deathrate[stepcnt])/LETFRRES,
	   (double)sptr->mig[stepcnt] * Country[Cdisp].scale_factor,
	   IDIV(sptr->deathrate[stepcnt][0], RATERES));
 }


/***********************************************************************
Find out if we are in the top or bottom window.
GLOBALS: Cdisp.
***********************************************************************/
int checkwindow(BigString windowname)
{
  if (strncmp(windowname, "top", 3) == 0)
    return(DISPLAY1);
  else if (strncmp(windowname, "bottom", 6) == 0)
    return(DISPLAY2);
  else
  {
    GSbell();
    GSelog(0, "In checkwindow, unknown window |%s|\n", windowname);
    return(-1);
  }
}


/***********************************************************************
Compute the bar graph boundary percentage and initialize populations.
***********************************************************************/
void setup_bargraph(void)
{
  int bound;
  long *maleptr = Display[Cdisp].male;
  long *femptr = Display[Cdisp].female;
  IPStatetype *sptr = IPState[Cdisp][Display[Cdisp].curr_sim];
  int stepcnt = sptr->stepcnt;
  int i;

  for (i=0; i<17; i++)
  {
    maleptr[i] = sptr->male_pop[stepcnt][i];
    femptr[i] = sptr->fem_pop[stepcnt][i];
  }
  bound = 20; /* start with 20% */
  if (sptr->totpop[stepcnt] != 0)
  {
    for(i=0; i<8; i++)
    {
      while (RESCALE(maleptr[2*i] + maleptr[2*i+1], 100,
		     sptr->totpop[stepcnt]) >= bound)
	bound += 10;
      while (RESCALE(femptr[2*i] + femptr[2*i+1], 100,
		     sptr->totpop[stepcnt]) >= bound)
	bound += 10;
    }
    while ((RESCALE(maleptr[16], 100, sptr->totpop[stepcnt]) >= bound) ||
	   (RESCALE(femptr[16], 100, sptr->totpop[stepcnt]) >= bound))
      bound += 10;
  }
  Display[Cdisp].bound = bound;
}


/***********************************************************************
Initialize data for a simulation.
***********************************************************************/
void init_sim_data(int d, int s) /* display, simulation number */
{
  IPStatetype *sptr;
  IPsaveyear_t *syptr;
  CountryInfo *cptr;
  int i;
  long tfr;  /* total fertility rate */

  /* activate buttons */
  if (!Display[d].modify_flag)
  {
    GSenablebutton(TFRname[d], "setfert", DRAW);
    GSenablebutton(Lifexname[d], "setlife", DRAW);
    GSenablebutton(Migname[d], "setmig", DRAW);
    GSenablebutton(Displayname[d], "pyrgraph", NO_DRAW);
  }
  else
  {
    GSenablebutton(Migname[d], "setmig", DRAW);
    GSdrawnamedwindow(Birthwinname[d], NO_BSTORE, NO_POPUP);
    GSdrawnamedwindow(Deathwinname[d], NO_BSTORE, NO_POPUP);
    GSdrawnamedwindow(Infantwinname[d], NO_BSTORE, NO_POPUP);
    GSenablebutton(Displayname[d], "pyrgraph", NO_DRAW);
  }
  sptr = IPState[d][s];
  syptr = IPsaveyear[d][s];
  cptr = &Country[d];
  sptr->stepcnt = 0;
  sptr->year = 1990;
  sptr->totpop[0] = cptr->male_1990_pop[0] + cptr->fem_1990_pop[0];
  for (i=0; i<17; i++)
  {
    sptr->male_pop[0][i] = cptr->male_1990_pop[i+1];
    sptr->fem_pop[0][i] = cptr->fem_1990_pop[i+1];
  }
  for (tfr=0, i=0; i<9; i++)
    tfr += sptr->birthrate[0][i] = cptr->tot_birthrate[i];
  tfr *= 5;  /* 5 year cohorts */
  for (i=0; i<18; i++)
    sptr->deathrate[0][i] = cptr->tot_deathrate[i];
  sptr->mig[0] = cptr->mig;

  sptr->tfr_targ_year[0] = 1990;
  syptr->tfr_targ_year[0] = 1990;
  sptr->tfr_targ_val[0] = (int)IDIV(tfr * (long)LETFRRES, 1000L * RATERES);
  sptr->lifex_targ_year[0] = 1990;
  syptr->lifex_targ_year[0] = 1990;
  sptr->lifex_targ_val[0] = cptr->life_expectancy;
  sptr->infmort_targ_year[0] = -1;
  sptr->infmort_targ_val[0] = -1;
  sptr->mig_targ_year[0] = 1990;
  syptr->mig_targ_year[0] = 1990;
  sptr->mig_targ_val[0] = cptr->mig;
  setup_bargraph();
}


/***********************************************************************
Initialize a country display's state.
***********************************************************************/
void init_display_data(int d) /* display number */
{
  int i;

  /* activate buttons  */
  GSenablebutton(Displayname[d], "simulate", DRAW);
  GSdisablebutton(Displayname[d], "unsimulate", DRAW);
  GSenablebutton(Displayname[d], "pyrgraph", NO_DRAW);
  Display[d].modify_flag = FALSE;
  Display[d].reset_permit = TRUE;
  Display[d].modify_permit = TRUE;
  Display[d].scenario_permit = TRUE;
  Display[d].alt_permit = TRUE;

  Display[d].curr_sim = 0;
  Display[d].maxpop = Country[d].male_1990_pop[0] + Country[d].fem_1990_pop[0];
  Display[d].intuppop = 0;
  init_sim_data(d, 0);
  for (i=1; i<NUM_SIMS; i++)
  {
    IPState[d][i]->stepcnt = -1;
    IPState[d][i]->totpop[0] = 0;
  }
}

/***********************************************************************
Convert an uppercase string to lower case.
***********************************************************************/
void tolowerstring(char * instring)
{
  int i;

  for (i=0; i<strlen(instring); i++)
     instring[i] = tolower(instring[i]);
}


/***********************************************************************
Read a country data file.
***********************************************************************/
boolean readcountry(int slot)
{
  FILE *countryfp;
  BigString filename;
  ReadinInfo rawcountry;

  if(slot < Numregions)
    GSolog("Selected region: = %s.\n", Country_list[slot].name);
  else
    GSolog("Selected country = %s.\n", Country_list[slot].name);
  GSolog ("Name           : Year --Population-- TFR LifeEx Migration IMR\n");
  tolowerstring(Country_list[slot].file);
  sprintf(filename, "%s%s", Datapath, Country_list[slot].file);
  if ((countryfp = GSopen((char *)filename, "r")) == NULL)
  {
    GSelog(0, "Unable to open country file |%s|\n", filename);
    GSalert(10, Use_bstore, "ctyerror", "");
    AlertNumber = 1;
    return(FALSE);
  }
  if (!GSgenreader(countryfp, IPreader, "IP_country", &rawcountry))
  {
    GSelog(0, "Unable to IPread country file |%s|\n", filename);
    GSalert(10, Use_bstore, "ctyerror", "");
    AlertNumber = 1;
    fclose(countryfp);
    return(FALSE);
  }
  fclose(countryfp);
  if (!DBconvert(&rawcountry, &Country[Cdisp]))
  {
    GSelog(0, "Unable to DBconvert country file |%s|\n", filename);
    GSalert(10, Use_bstore, "ctyerror", "");
    AlertNumber = 1;
    return(FALSE);
  }
  return(TRUE);
}


/***********************************************************************
Display the current state of the population pyramid.
It also returns the current country population total.
***********************************************************************/
void display_bar_graph(int d) /* which display */
{
  int x, y, wd, ht, bh, dum;
  GSColor color, barcolor;
  int i;
  long male, female;  /* number of pixels in bar */
  IPStatetype *sptr = IPState[d][Display[d].curr_sim];
  long *maleptr = sptr->male_pop[sptr->stepcnt];
  long *femptr = sptr->fem_pop[sptr->stepcnt];
  long scale, safescale;
  int labelx;
  int bound = Display[d].bound;

/*  box below pyramid graph should be a field in the interface file. -- MRL  */
  GSsetcurrentnamedwindow(Displayname[d]);
  GSgetfieldrect("PYRGRAPH", &x, &y, &wd, &ht, &color);
  GSgetfieldrect("BARS", &dum, &dum, &dum, &bh, &color);
  GSwclearrect(x, y+12, wd, ht-15);
  GSwclearrect(x-8, y+ht+1, wd+30, 16);

  GSsetfont(LARGE);
  GSsetcolor(color);
  GSwwriteint(2, x-8, y+ht+12, bound);
  GSwwriteint(2, x+wd/4-10, y+ht+12, bound/2);
  GSwwritenamedmsg(x+wd/2-6, y+ht+11, "Oper");
  GSwwriteint(2, x+3*wd/4-6, y+ht+12, bound/2);
  GSwwriteint(2, x+wd-6, y+ht+12, bound);

  wd /= 2;
  y = y - 5 - bh;
  GSsetfont (SMALL);   /* for cohort labels */
  GSsetcolor(color);
  if (sptr->totpop[sptr->stepcnt] == 0)
  {
    scale = MAXINT;
    safescale = 1;
  }
  else if (sptr->totpop[sptr->stepcnt] < 50000000L)
  {
    scale = sptr->totpop[sptr->stepcnt] * bound;
    safescale = 1;
  }
  else
  { /* Attempt to recognize a multiplication overflow.
       This is extremly rare, but this is still a kludge -- CAS */
    scale = IDIV(sptr->totpop[sptr->stepcnt], 1000) * bound;
    safescale = 1000;
  }

  for (i=0; i<9; i++)
  {
    barcolor = GSgetlistcolor(1, i % 4);
    if (i == 8)
    {
      male = RESCALE(maleptr[16], wd*100, scale);
      female = RESCALE(femptr[16], wd*100, scale);
    }
    else
    {
      male = RESCALE(maleptr[2*i]+maleptr[2*i+1], wd*100, scale);
      female = RESCALE(femptr[2*i]+femptr[2*i+1], wd*100, scale);
    }
    male /= safescale;
    female /= safescale;

    GSsetcolor(barcolor);
    GSwfillrect(x + wd, y + ht - (bh-1) * i, (int)female, bh);
    GSwfillrect(x + wd - (int)male, y + ht - (bh-1) * i, (int)male, bh);
    GSsetcolor(color);
    if ((int)female <= 0)
    { /*--- GSwdrawrect will have negative width, so draw line instead ---*/
      GSwdrawline(x+wd, y+ht-(bh-1)*i, x+wd, y+ht-(bh-1)*i+bh);
    }
    else
    {
      GSwdrawrect(x + wd, y + ht - (bh-1) * i, (int)female, bh);
    }
    if ((int)male <= 0)
    { /*--- GSwdrawrect will have negative width, so draw line instead ---*/
      GSwdrawline(x+wd-(int)male, y+ht-(bh-1)*i, x+wd-(int)male,
                  y+ht-(bh-1)*i+bh);
    }
    else
    {
      GSwdrawrect(x + wd - (int)male, y + ht - (bh-1) * i, (int)male, bh);
    }
    /* display cohort label */
    if ((male < 19) || (female < 19) || (i==8)) /* 80+ always to right */
      if (female > 38)
	labelx = wd + 3; /* shift text into female bar */
      else
	labelx = wd + (int)female + 5;  /* shift text right of female bar */
    else if (i == 0) /* shorter label */
      labelx = wd - 8;
    else
      labelx = wd - 15;
    GSsetfont (SMALL); /* for cohort labels */
    GSsetcolor(color);
#ifdef MSDOS
    GSwwritemsg(x + labelx, y + ht + 9 - i*(bh-1), Clabel[i]);
#else
    GSwwritemsg(x + labelx - 3, y + ht + 11 - i*(bh-1), Clabel[i]);
#endif
  } /* for */
  Display[d].bound = bound;
}


/***********************************************************************
Display the population graph for country display "d".
***********************************************************************/
void display_pop_graph(int d) /* display number */
{
  int s; /* simulation number */
  int p; /* plot point number */
  int x, y, ht, wd; /* field rectangle */
  GSColor color;
  long currpop;
  int height, ht1, ht2, sect;
  int cnt, i;
  int lastyear;
  long intuppop;
  IPStatetype *sptr;  /* a simulation descriptor */
  double dpop;

  /* compute and draw scale labels */
  intuppop = Display[d].intuppop;
  if (intuppop < Display[d].maxpop)
    Display[d].intuppop = intuppop = IDIV(Display[d].maxpop * 3, 2);
  for (cnt=0; intuppop>10L; cnt++)
    intuppop /= 10L;
  if (ODD(intuppop)) intuppop++;  /* make it even */
  for (i=1; i<=cnt; i++)
    intuppop *= 10L;
  Display[d].intuppop = intuppop; /* so that maxpop can't be bigger
				     than truncated intuppop */

  GSgetfieldrect("MINPOP", &x, &y, &wd, &ht, &color);
  GSwclearrect(x, y, wd, ht);
  GSsetfont(SMALL);
  GSsetcolor(color);
  GSwwriteint(3, x, y+SMALL, 0L);

  GSgetfieldrect("MAXPOP", &x, &y, &wd, &ht, &color);
  GSsetcolor(color);
  GSwclearrect(x, y, wd, ht);
  y += SMALL;
  dpop = (double)intuppop * (double)Country[d].scale_factor;
  if (dpop < 990000.0)
  {
    GSwwriteint(3, x, y, (long)(dpop/1000.0));
    GSwwritemsg(x, y+SMALL, "THO");
  }
  else if (dpop < 990000000.0)
  {
    GSwwriteint(3, x, y, (long)(dpop/1000000.0));
    GSwwritemsg(x, y+SMALL, "MIL");
  }
  else
  {
    GSwwriteint(3, x, y, (long)(dpop/1000000000.0));
    GSwwritemsg(x, y+SMALL, "BIL");
  }

  GSgetfieldrect("MIDPOP", &x, &y, &wd, &ht, &color);
  GSsetfont(SMALL);
  GSsetcolor(color);
  GSwclearrect(x, y, wd, ht);
  y += SMALL;
  dpop = (double)intuppop/2.0 * (double)Country[d].scale_factor;
  if (dpop < 990000.0)
  {
    GSwwriteint(3, x, y, (long)(dpop/1000.0));
    GSwwritemsg(x, y+SMALL, "THO");
  }
  else if (dpop < 990000000.0)
  {
    GSwwriteint(3, x, y, (long)(dpop/1000000.0));
    GSwwritemsg(x, y+SMALL, "MIL");
  }
  else
  {
    GSwwriteint(3, x, y, (long)(dpop/1000000000.0));
    GSwwritemsg(x, y+SMALL, "BIL");
  }

  /* Draw year labels */
  GSgetfieldrect("GRAPHYEAR", &x, &y, &wd, &ht, &color);
  GSsetfont(SMALL);
  GSsetcolor(color);
  lastyear = 1990;
  for (i=0; i<=Display[d].curr_sim; i++)
    if (IPState[d][i]->year > lastyear)
      lastyear = IPState[d][i]->year;

  for (i=0; i <= (lastyear-1990) / 5; i+=2)
    GSwwriteint(3, x + i * 17, y+SMALL, (long) (i*5 + 1990));

  GSwclearrect(x + i*17, y, wd-i*17, ht);

  /* draw doubling bars */
  GSgetfieldrect("SIMGRAPH", &x, &y, &wd, &ht, &color);
  GSsetcolor(color);
  GSwclearrect(x, y, wd, ht);
  wd-=1; ht -= 3;  /*  so graph borders aren't overwritten */
  for (currpop=(Country[d].male_1990_pop[0]+Country[d].fem_1990_pop[0]);
       currpop<=Display[d].maxpop; currpop *= 2)
  {
    height = (int)RESCALE(currpop, ht, intuppop);
    GSwdrawline(x, y+ht-height, x+wd-1, y+ht-height);
  }

  /* plot graph */
  for (s=0; s<NUM_SIMS; s++)
  {
    sptr = IPState[d][s];
    color = GSgetlistcolor(0, s);
    ht2 = (int)RESCALE(sptr->totpop[0], ht, intuppop);
    sect = wd / (NUM_POINTS-1);
    GSsetlinesize(THICK);
    for (p=1; p<=sptr->stepcnt; p++)
    {
      ht1 = ht2;
      ht2 = (int)RESCALE(sptr->totpop[p], ht, intuppop);
      GSsetcolor(color);
      GSwdrawline(x + (p-1)*sect, y+ht-ht1,
		  x + p*sect, y+ht-ht2);
    } /* for p */
    GSsetlinesize(THIN);
  } /* for s */
GSgetfieldrect("SIMGRAPH", &x, &y, &wd, &ht, &color);
GSsetcolor(color);
GSwdrawrect (x, y, wd, ht);
GSwdrawrect (x-1, y-1, wd+2, ht+2);
}


/***********************************************************************
Combine two cohorts (if necessary) to produce a 10 year birthrate.
***********************************************************************/
long rebuild_birthrate(int num, IPStatetype *sptr, long *female, int stepcnt)
{
  long sum, temp;

  if (num == 4) /* shown as 50-59, but calculated as 50-54 */
    sum = IDIV(sptr->birthrate[stepcnt][8], RATERES);
  else
  { /* must take weighted avg for two cohorts grouped together */
    temp = female[2*num+2] + female[2*num+3];
    if (temp != 0)
    {
      sum = RESCALE(sptr->birthrate[stepcnt][2*num], female[2*num+2], temp) +
            RESCALE(sptr->birthrate[stepcnt][2*num+1], female[2*num+3], temp);
      sum = IDIV(sum, RATERES);
    }
    else
      sum = 0;
  }
  return(sum);
}


/***********************************************************************
Combine two cohorts (if necessary) to produce a 10 year deathrate.
***********************************************************************/
long rebuild_deathrate(int num, IPStatetype *sptr,
                       long *male, long *female, int stepcnt)
{
  long sum, temp;

  if (num == 8) /* 80+ */
    sum = IDIV(sptr->deathrate[stepcnt][17], RATERES);
  else
  { /* must take weighted avg for two cohorts grouped together */
    temp = male[2*num] + male[2*num+1] + female[2*num] + female[2*num+1];
    if (temp != 0)
    {
      sum = RESCALE(sptr->deathrate[stepcnt][2*num+1],
                    male[2*num] + female[2*num], temp) +
            RESCALE(sptr->deathrate[stepcnt][2*num+2],
                    male[2*num+1] + female[2*num+1], temp);
      sum = IDIV(sum, RATERES);
    }
    else
      sum = 0;
  }
  return(sum);
}


/***********************************************************************
Redraw all simulation information for a country display.
***********************************************************************/
void update_display(int d) /* display number */
{
  int s; /* simulation number */
  int x, y, wd, ht, bh;  /* field rectangle */
  GSColor color, plotColor;
  IPStatetype *sptr;
  int tfr;
  int i;
  int dum;  /* for parameters not used */
  long total;
  long *male, *female;
  int stepcnt;  /* current simulations's step */

  GSsetcurrentnamedwindow(Displayname[d]);
  if(Display[d].curr_sim < 0)
    return;

  sptr = IPState[d][Display[d].curr_sim];
  stepcnt = sptr->stepcnt;

  /* Write country name */
  GSgetfieldrect ("CTYNAME", &x, &y, &wd, &ht, &color);
  GSwclearrect (x, y+1, wd, ht);
  GSsetfont(LARGE);
  GSsetcolor(color);
  for (i=0; i<strlen (Country[d].country_name); i++)
     Country[d].country_name[i] = toupper(Country[d].country_name[i]);
  GSwwritemsg(x, y+LARGE, Country[d].country_name);

  /* Display the population graph */
  display_pop_graph(d);

  /* display birth and death rates for each age group */
  GSgetfieldrect ("BARS", &dum, &dum, &dum, &bh, &color);
  GSgetfieldrect ("BIRTHRATE", &x, &y, &wd, &ht, &color);
  GSwclearrect (x, y, wd, ht);
  GSsetfont (SMALL);
  GSsetcolor(color);
  male = sptr->male_pop[stepcnt];
  female = sptr->fem_pop[stepcnt];
  for (i=0; i<5; i++)
    GSwwriteint(3, x, y + SMALL + (4-i)*(bh-1),
                rebuild_birthrate(i, sptr, female, stepcnt));
  GSgetfieldrect ("DEATHRATE", &x, &y, &wd, &ht, &color);
  GSwclearrect (x, y, wd, ht);
  GSsetfont (SMALL);
  GSsetcolor(color);
  for (i=0; i<9; i++)
    GSwwriteint(3, x, y + SMALL + (8-i)*(bh-1),
                rebuild_deathrate(i, sptr, male, female, stepcnt));

/*------------------------------------------------------------------------------
  MAY PUT THIS BACK IN LATER
  for (i=0; i<9; i++)
  {
    /--- do button disabling and enabling ---/
    sprintf(tmpstr, "%s%d%s", "Death", i, "m");
    GSgetbuttonspecs(Deathwinname[d], tmpstr, NULL, NULL, NULL, NULL, &status);
    
    if ((sptr->deathrate[stepcnt][2*i+1] <= 0) &&
        ((i == 8) || (sptr->deathrate[stepcnt][2*i+2] <= 0)))
    {
      if (status != DISABLED)
      {
        GSdisablebutton(Deathwinname[d], tmpstr, DRAW);
      }
    }
    else
    {
      if (status != ENABLED)
      {
        GSenablebutton(Deathwinname[d], tmpstr, DRAW);
      }
    }

    sprintf(tmpstr, "%s%d%s", "Death", i, "p");
    GSgetbuttonspecs(Deathwinname[d], tmpstr, NULL, NULL, NULL, NULL, &status);

    if ((sptr->deathrate[stepcnt][2*i+1] >= MAXRATE) &&
        ((i == 8) || (sptr->deathrate[stepcnt][2*i+2] >= MAXRATE)))
    {
      if (status != DISABLED)
      {
        GSdisablebutton(Deathwinname[d], tmpstr, DRAW);
      }
    }
    else
    {
      if (status != ENABLED)
      {
        GSenablebutton(Deathwinname[d], tmpstr, DRAW);
      }
    }
  } /--- end of button disabling, enabling ---/
-------------------------------------------------------------------------------*/

  /* display infant death rate */
  GSgetfieldrect ("INFRATE", &x, &y, &wd, &ht, &color);
  GSwclearrect (x, y, wd, ht);
  GSsetfont (SMALL);
  GSsetcolor(color);
  GSwwriteint(3, x, y+SMALL, IDIV(sptr->deathrate[stepcnt][0], RATERES));

  /* Display current TFR */
  GSsetcurrentnamedwindow(TFRname[d]);
  tfr = get_TFR(sptr->birthrate[stepcnt]);
  GSgetfieldrect("TFR", &x, &y, &wd, &ht, &color);
  GSwclearrect(x, y, wd, ht);
  GSsetfont (SMALL);
  GSsetcolor(color);
  GSwwritereal (3, 1, x, y-1+SMALL, (double)tfr/LETFRRES);

  /* Display current life expectancy */
  GSsetcurrentnamedwindow (Lifexname[d]);
  GSgetfieldrect("LIFEX", &x, &y, &wd, &ht, &color);
  GSwclearrect(x, y, wd, ht);
  GSsetfont (SMALL);
  GSsetcolor(color);
  GSwwritereal(4, 1, x, y-1+SMALL,
	       (double)get_life_expect(sptr->deathrate[stepcnt])/LETFRRES);

  /* Display current net migration */
  GSsetcurrentnamedwindow(Migname[d]);
  GSgetfieldrect("MIG", &x, &y, &wd, &ht, &color);
  GSwclearrect(x, y, wd, ht);
  GSsetfont (SMALL);
  GSsetcolor(color);
  GSwwritereal (10, 0, x, y-1+SMALL,
      (double)sptr->mig[stepcnt] * Country[d].scale_factor);

  /* Display the bars in the population pyramid */
  display_bar_graph(d);
  plotColor = GSgetlistcolor(0, Display[d].curr_sim);

  GSgetfieldrect("PYRYEAR", &x, &y, &wd, &ht, &color);
  GSwclearrect(x, y, wd, ht);
  GSsetfont(SMALL);
  GSsetcolor(plotColor);
  GSwwriteint(3, x, y+SMALL, sptr->year);

  /* Display current year population */
  GSgetfieldrect("SIMYEAR", &x, &y, &wd, &ht, &color);
  GSwclearrect(x, y, wd, ht);
  GSwwriteint(3, x, y+SMALL, sptr->year);

  /*  cover up birth/death rate buttons if necessary  */
  if (!Display[d].modify_flag)
  {
    GSsetcurrentnamedwindow(Displayname[d]);
    GSgetfieldrect("BRATE", &x, &y, &wd, &ht, &color);
    GSwclearrect(x, y, wd, ht);
    GSgetfieldrect("DRATE", &x, &y, &wd, &ht, &color);
    GSwclearrect(x, y, wd, ht);
    GSgetfieldrect("IRATE", &x, &y, &wd, &ht, &color);
    GSwclearrect(x, y, wd, ht);
  }

  /* Display initial (1990) population */
  GSsetcurrentnamedwindow(Infowinname[d]);
  GSgetfieldrect("INITPOP", &x, &y, &wd, &ht, &color);
  GSwclearrect(x, y, wd, ht);
  GSsetfont(SMALL);
  GSsetcolor(color);
  GSwwritecommanum(x, y+SMALL, (double)Country[d].scale_factor *
	    (double)(Country[d].male_1990_pop[0]+Country[d].fem_1990_pop[0]));


  for (s=0; s<NUM_SIMS; s++)
    if (IPState[d][s]->stepcnt >= 0)
    {
      plotColor = GSgetlistcolor(0, s);
      GSgetfieldrect ("INFOWIND", &x, &y, &wd, &ht, &color);
      if (s == 0)
        GSwclearrect (x, y, wd, ht*3);
      else
        y += ht * s;
      GSsetfont(SMALL);
      GSsetcolor(plotColor);
      GSwwritenamedmsg(x, y + SMALL, "cyear");
      GSwwritenamedmsg(x, y + 2 * SMALL, "pop");
      x += 23;
      wd -= 23;
      GSwclearrect(x, y, wd, ht);
      GSwwriteint(3, x+8, y+SMALL, (long)IPState[d][s]->year);
      male = IPState[d][s]->male_pop[IPState[d][s]->stepcnt];
      female = IPState[d][s]->fem_pop[IPState[d][s]->stepcnt];
      for (total=0, i=0; i<17; i++)
        total += male[i] + female[i];
      GSwwritecommanum(x, y+2*SMALL,
                       (double)total*(double)Country[d].scale_factor);
      if (s == Display[d].curr_sim)
      {
        GSsetcurrentnamedwindow(Displayname[d]);
        GSgetfieldrect("SIMPOP", &x, &y, &wd, &ht, &color);
        GSwclearrect(x, y, wd, ht);
        GSwwritecommanum(x, y+SMALL,
                         (double)total*(double)Country[d].scale_factor);
        GSsetcurrentnamedwindow(Infowinname[d]);
      }
    } /* If simulation active */
}


/***********************************************************************
Redraw all simulation information to the screen.  Typically done
in response to clearing an overlapping window (i.e., set boxes).
***********************************************************************/
void redraw_all(void)
{
  int x, y;
  GSColor color;
  BigString msgstring;
  static boolean first = TRUE;

  GSredrawnamedwindow("root");
  GSdisplayversion (VersionFile);
  redraw_display(DISPLAY1);
  update_display(DISPLAY1);
  redraw_display(DISPLAY2);
  update_display(DISPLAY2);
  if (first)
  {
    first = FALSE;
  }
  else
  {
    GSdisplayhelpmessage (-1);
  }
  if (SetFertActive)
    display_fert_win(DRAW);
  if (SetLifeExActive)
    display_lifeex_win(DRAW);
  if (SetMigActive)
    display_mig_win(DRAW);

  if (MapSelect)
  {
    GSredrawnamedwindow ("map");
    draw_selection_window();
  }
  if (QuickStartActive)
  {
    GSredrawnamedwindow ("instruction");
    drawintroscreen();
  }
  if (HelpWinActive)
    GSdohelpscreens (Helpfile, CurrPage, Use_bstore, -1);
  if (QuitWinActive)
  {
    GSredrawnamedwindow ("quit");
  }
  switch(AlertNumber)
  {
    case 1:
      GSalert(10, Use_bstore, "ctyerror", "");
      break;
    case 2:
      GSalert(10, Use_bstore, "loadalert1", "loadalert2", "loadalert3",
              "loadalert4", "");
      break;
    case 3:
      GSalert(96, Use_bstore, "introalert1", "introalert2", "introalert3",
              "introalert4", "introalert5", "introalert6",  "");
      break;
    default:
      break;
  }

  if (DumpResultsActive)
  {
    GSredrawnamedwindow("alert");
    GSsetcolor(GSconvertcolor("black"));
    GSsetfont(LARGE);

    if (DumpSuccess)
    {
      sprintf(msgstring, "The screen was saved to file:    %s", DumpFileName);
      GSwwritemsg(6, 30, msgstring);
    }
    else
    {
      sprintf(msgstring, "An error occurred saving the screen.");
      GSwwritemsg(6, 15, msgstring);
      sprintf(msgstring, "No file was saved.");
      GSwwritemsg(6, 30, msgstring);
      sprintf(msgstring, "Check disk space and file permissions.");
      GSwwritemsg(6, 45, msgstring);
      GSbell();
    }
  }

  if (DumpWinActive)
  {
    GSredrawnamedwindow ("dump");

    highlightfield("dump", "filetype", GSconvertcolor("lgray"),
                   GSconvertcolor("black"), GSconvertcolor("white"));
    GSgetfieldrect("filetype", &x, &y, NULL, NULL, &color);
    GSsetcolor(color);
    GSsetfont(LARGE);
    switch (GraphFileFmt)
    {
      case BMP_16:
        GSwwritenamedmsg(x+5, y+SMALL+2, "bmp16msg");
        break;
      case GIF_16:
        GSwwritenamedmsg(x+5, y+SMALL+2, "gif16msg");
        break;
      case PS_GRAY:
        GSwwritenamedmsg(x+5, y+SMALL+2, "psgraymsg");
        break;
      case PS_COLOR:
        GSwwritenamedmsg(x+5, y+SMALL+2, "pscolormsg");
        break;
      case NO_FILE: default:
        GSwwritenamedmsg(x+5, y+SMALL+2, "nofilemsg");
        break;
    }
  }
}

/***********************************************************************
Redraw all simulation display information to the screen.  Typically done
in response to clearing an overlapping window (i.e., set boxes).
***********************************************************************/
void redraw_display (int d)
{
  GSredrawnamedwindow (Displayname[d]);
  GSredrawnamedwindow(Birthwinname[d]);
  GSredrawnamedwindow(Deathwinname[d]);
  GSredrawnamedwindow(Infantwinname[d]);
/*  GSdrawnamedwindow(TFRname[d], NO_BSTORE, NO_POPUP);
  GSdrawnamedwindow (Lifexname[d], NO_BSTORE, NO_POPUP);
  GSdrawnamedwindow (Migname[d], NO_BSTORE, NO_POPUP);
  GSdrawnamedwindow(Infowinname[d], NO_BSTORE, NO_POPUP);*/
  GSredrawnamedwindow(TFRname[d]);
  GSredrawnamedwindow (Lifexname[d]);
  GSredrawnamedwindow (Migname[d]);
  GSredrawnamedwindow(Infowinname[d]);
}


/* Increment or decrement target total fertility rate value */
boolean changefert(int changeval)
{
  int x, y, ht, wd;
  GSColor color;
  IPStatetype *sptr = IPState[Cdisp][Display[Cdisp].curr_sim];
  /* The following must be a pointer so that value gets changed in structure */
  int *vptr = &sptr->tfr_targ_val[sptr->stepcnt];

  *vptr += changeval*10;
  if (*vptr < 50)
  {
    *vptr = 50;
    GSbell();
  }
  else if (*vptr > 990)
  {
    *vptr = 990;
    GSbell();
  }
  GSgetfieldrect("SETFERT", &x, &y, &wd, &ht, &color);
  GSwclearrect (x, y, wd, ht);
  GSsetfont (LARGE);
  GSsetcolor(color);
  GSwwritereal(3, 1, x, y+LARGE, (double)(*vptr)/LETFRRES);
  GSclick();
  return(TRUE);
}


/*-------------------------------------------------------------------------------
  Increment or decrement target life expectancy value
-------------------------------------------------------------------------------*/
boolean changelifex (
  int changeval
)
{
  int         x, y, ht, wd;
  GSColor     color;
  IPStatetype *sptr = IPState[Cdisp][Display[Cdisp].curr_sim];
  /* The following must be a pointer so that value gets changed in structure */
  int         *vptr = &sptr->lifex_targ_val[sptr->stepcnt];

  *vptr += changeval*10;
  if (*vptr < 2000)
  {
    *vptr = 2000;
    GSbell();
  }
  else if (*vptr > 9990)
  {
    *vptr = 9990;
    GSbell();
  }
  GSgetfieldrect("SETLIFE", &x, &y, &wd, &ht, &color);
  GSwclearrect (x, y, wd, ht);
  GSsetfont (LARGE);
  GSsetcolor(color);
  GSwwritereal(3, 1, x, y+LARGE, (double)(*vptr)/LETFRRES);
  GSclick();
  return(TRUE);
}


/*-------------------------------------------------------------------------------
  Increment or decrement target migration value
-------------------------------------------------------------------------------*/
boolean changemig(int changeval)
{
  int         x, y, ht, wd;
  GSColor     color;
  IPStatetype *sptr = IPState[Cdisp][Display[Cdisp].curr_sim];
  /* The following must be a pointer so that value gets changed in structure */
  long        *vptr = &sptr->mig_targ_val[sptr->stepcnt];
  long        val;

  if (Country[Cdisp].scale_factor == 1)
    val = 1000;
  else if (sptr->totpop[sptr->stepcnt] < 1000000L)
    val = 10;
  else
    val = 100;

  *vptr += changeval * val;
  if (*vptr < -100000L)
  {
    *vptr = -100000L;
    GSbell();
  }
  else if (*vptr > 100000L)
  {
    *vptr = 100000L;
    GSbell();
  }
  GSgetfieldrect("SETMIG", &x, &y, &wd, &ht, &color);
  GSwclearrect (x, y, wd, ht);
  GSsetfont (LARGE);
  GSsetcolor(color);
  GSwwriteint(10, x, y+LARGE, *vptr * Country[Cdisp].scale_factor);
  GSclick();
  return(TRUE);
}


/*-------------------------------------------------------------------------------
  Increment or decrement target year for TFR, life exp, or migration
-------------------------------------------------------------------------------*/
boolean changeyear (
  int         changeval,
  IPStatetype *sptr,
  int         *yeartarget,
  char        *yearfield)
{
  int     x, y, ht, wd;
  GSColor color;

  if (((changeval > 0) && (*yeartarget <= 2045)) ||
      ((changeval < 0) && (*yeartarget > sptr->year)))
    *yeartarget += changeval;
  else
    GSbell();
  GSgetfieldrect(yearfield, &x, &y, &wd, &ht, &color);
  GSwclearrect(x, y, wd, ht);
  GSsetfont (LARGE);
  GSsetcolor(color);
  GSwwriteint(4, x, y+LARGE, (long)(*yeartarget));
  GSclick();
  return(TRUE);
}


boolean startsim(int flag)
{
  int s; /* current sim number */
  IPStatetype *sptr;

  if (Display[Cdisp].curr_sim+1 == NUM_SIMS)
  {
    GSelog(0, "ERROR - tried to init alt simulation #%d\n",
	   Display[Cdisp].curr_sim+1);
    return(FALSE);
  }
  s = ++Display[Cdisp].curr_sim;
  Display[Cdisp].modify_permit = TRUE; /* allow toggling modify rates */
  if (s+1 == NUM_SIMS)
  {
    Display[Cdisp].scenario_permit = FALSE;
    Display[Cdisp].alt_permit = FALSE;
  }
  GSdisablebutton(Displayname[Cdisp], "unsimulate", DRAW);
  GSenablebutton(Displayname[Cdisp], "simulate", DRAW);
  init_sim_data(Cdisp, s);
  if (flag == 2000)
  {
    sptr = IPState[Cdisp][s];
	sptr->tfr_targ_year[0] = 2000;
    sptr->tfr_targ_val[0] = Country[Cdisp].tot_fertility_2000;
    sptr->lifex_targ_year[0] = 2000;
    sptr->lifex_targ_val[0] = Country[Cdisp].life_expect_2000;
    sptr->infmort_targ_year[0] = 2000;
    sptr->infmort_targ_val[0] = Country[Cdisp].infant_deaths_2000;
  }
  update_display(Cdisp);
  if (!InWalkThru)
    GSdisplayhelpmessage(4);
  return(TRUE);
}


void reset_screen(boolean flag)
{
  GSremovetopwindow();
  if (!Use_bstore)
    redraw_all();
  else if (flag)
    update_display(Cdisp);
  GSdisplayhelpmessage(ParmSavemsg);
}


/* Change map scale */
boolean changemapscale(int changeval)
{
  int x, y, wd, ht;
  int lx, ly, lwd, lht;
  GSColor color;

  if (((MapScale + changeval) < 1) || ((MapScale + changeval) > 31))
    return(TRUE);
  MapScale += changeval;
  GSgetfieldrect("SMAP", &x, &y, &wd, &ht, &color);
  GSgetfieldrect("LMAP", &lx, &ly, &lwd, &lht, &color);
  GSdrawxorbox(Xorx+x, Xory+y, Xorwd, Xorht); /* Clear old Xor box */
  GSsetcolor(color);
  GSwfillrect(lx, ly, lwd, lht);
  compute_xorbox(Xorx+Xorwd/2, Xory+Xorht/2, MapScale, wd, ht, lwd, lht);
  GSdrawxorbox(Xorx+x, Xory+y, Xorwd, Xorht);
  GSgetfieldrect("MSCALEBACK", &x, &y, &wd, &ht, &color);
  GSsetcolor(color);
  GSwfillrect(x, y, wd, ht);
  GSgetfieldrect("MSCALE", &x, &y, &wd, &ht, &color);
  GSsetfont(LARGE);
  GSsetcolor(color);
  GSwwriteint(3, x, y+LARGE, (long)MapScale);
  GSwdrawpict(&Countrypic, lx, ly, lwd, lht, 2*Xorx, 2*Xory, MapScale,
              code2color);
  return(TRUE);
}


/*************************************************************************
   Linear search.  Returns the index of the first country/region in the
   list array that matches the letters in key.
   Returns -1 if key is not in list.
**************************************************************************/
int find (Clist *list, char *key, int first, int last)
{
  int i;
  int CompareResult;
  BigString tempstring;
  int len;

  len = strlen(key);
  if (len > 0)
    for (i=first; i<last; i++)
    {
      strncpy(tempstring, list[i].name, len);
      tempstring[len]='\0';
	  CompareResult=GSstricmp( key, tempstring );
	  if ( CompareResult == 0 )
	    return(i);
	  else if ( CompareResult < 0 )
	    return (-1);
    }

  return(-1);
}

void display_searchstring(void)
{
  int x, y, wd, ht;
  GSColor color;
  int lx, ly, lwd, lht;
  GSColor lcolor;
  BigString outstring;

  GSgetfieldrect("searchtext", &x, &y, &wd, &ht, &color);
  GSgetfieldrect("scrollpicklabel", &lx, &ly, &lwd, &lht, &lcolor);

  GSsetcolor(color);
  GSwfillrect(x, y, wd, ht);
  GSsetfont (LARGE);
  GSsetcolor(lcolor);
  strcpy(outstring, searchstring);
  outstring[MAX_OUT_SEARCH_LN] = '\0';
  GSwwritemsg (x+5, y+LARGE-2, outstring);

}

/*************************************************************************
processes a character of the searchtext string.
**************************************************************************/
boolean ProcessSearchtext(char ch)
{
  int x, y, wd, ht;
  GSColor color;
  int lx, ly, lwd, lht;
  GSColor lcolor;
  int length;
  int index;
  int temp;
  BigString currentwindow;
  static boolean moveflag = FALSE;
  int slot;
  int i;
  int num_not_displayed=0;

  GSgetfieldrect("scrollpicklabel", &lx, &ly, &lwd, &lht, &lcolor);
  GSgetcurrentwindowname(currentwindow);
  GSsetcurrentnamedwindow("map");


  if ( (isalnum(ch))
	   || (ch == '.')
	   || (ch == '-')
	   || (ch == ' ') )
  {
	length = strlen(searchstring);
	if (length < BIGSTRINGLEN)
	{
	  searchstring[length] = ch;
	  searchstring[length+1] = '\0';
	}
  }
  else if ( (ch == '\b') && (strlen(searchstring) > 0) )
  {
	searchstring[strlen(searchstring)-1] = '\0';
  }

  display_searchstring();

  index = find(Country_list, searchstring, 0, Realnumcountries+Realnumregions );

  if (ch == '\r')
    if (index != -1)
    {
      /* subtract the number of countries that are not displayed */
      for (i = 0; i<index; i++)
   	    if (!Country_list[i].display)
          num_not_displayed++;
      scrollpickclick_func(index-num_not_displayed);
    }

  if ((index == -1) && (searchstring[0] != '\0') )
  {
    GSbell();
	return(FALSE);
  }
  else
  {
    temp = index;
    for (i=0; i<=temp; i++)
      if (!Country_list[i].display)
        index--;

	if (++index < Picklistpage)
	  scrollup(Picklistpage-index);
	else
	  scrolldn(index-Picklistpage);
  }

  GSgetfieldrect("scrollpickfield", &x, &y, &wd, &ht, &color);

  slot = (index < ScrollPicknumpages ) ? 0 : (index - ScrollPicknumpages);
  if (moveflag)
  /* change the mouse position slightly each time so field will rehighlight. */
	GSwsetmouseposition(x+lwd/2, y+slot*lht+lht/2);
  else
	GSwsetmouseposition(x+lwd/2+1, y+slot*lht+lht/2);
  moveflag = !moveflag;

  GSsetcurrentnamedwindow(currentwindow);

  return(TRUE);
}

/****************************************************************************
handlesearch
  passes the chararcter sent in the x parameter to ProcessSearchtext
  if called with status == KEY_PRESS
****************************************************************************/
boolean handlesearch(int x, int y, char ch)
{
	x = x;		/* to make the compiler happy */
	y = y;  	/* to make the compiler happy */

    return( ProcessSearchtext(ch) );
}


/***************************************************************************
  Save the current simulation state to filename
***************************************************************************/
boolean savenamedfile (char *filename)
{
  FILE *fptr;
  int i, j;

  if ((fptr = GSopen (filename, "wb")) == NULL)
    {
    GSelog (0, "Error saving current simulation.\n");
    return (FALSE);
    }
  for (i=0; i < NUM_DISPLAYS; i++)
    {
    if (fwrite (&(Display[i]), sizeof (DisplayType), 1, fptr) != 1)
      GSelog (0, "Error while saving program state.\n");
    if (fwrite (&(Country[i]), sizeof (CountryInfo), 1, fptr) != 1)
	  GSelog (0, "Error while saving program state.\n");
    for (j=0; j < NUM_SIMS; j++)
      if (fwrite (IPState[i][j], sizeof (IPStatetype), 1, fptr) != 1)
        GSelog (0, "Error while saving program state.\n");
    }
  GSsavewindows (fptr);
  fclose (fptr);
  return (TRUE);
}



/***************************************************************************
  Load the simulation state from filename
***************************************************************************/
boolean loadnamedfile (char *filename)
{
  FILE *fptr;
  int i, j;

  if ((fptr = GSopen (filename, "rb")) == NULL)
  {
    GSelog (0, "Error loading current simulation; filename: %s\n", filename);
    return (FALSE);
  }
  for (i=0; i < NUM_DISPLAYS; i++)
  {
    if (fread (&(Display[i]), sizeof (DisplayType), 1, fptr) != 1)
      GSelog (0, "Error loading a saved simulation.\n");
    if (fread (&(Country[i]), sizeof (CountryInfo), 1, fptr) != 1)
      GSelog (0, "Error loading a saved simulation.\n");
    for (j=0; j < NUM_SIMS; j++)
      if (fread (IPState[i][j], sizeof (IPStatetype), 1, fptr) != 1)
	GSelog (0, "Error loading a saved simulation.\n");
  }
  GSloadwindows (fptr);
  fclose (fptr);

  redraw_all ();
  if (!InWalkThru)
    GSdisplayhelpmessage(6);
  return (TRUE);
}

/*-------------------------------------------------------------------------------
  Draw Set TFR box.
-------------------------------------------------------------------------------*/
void display_fert_win (
  boolean redraw /*--- if TRUE, redraw window and its contents ---*/
)
{
  int         x, y, wd, ht;
  GSColor     color;
  IPStatetype *sptr;
  IPsaveyear_t *syptr;

  sptr = IPState[Cdisp][Display[Cdisp].curr_sim];
  syptr = IPsaveyear[Cdisp][Display[Cdisp].curr_sim];

  if (redraw)
    GSredrawnamedwindow("setfert");

  GSgetfieldrect("SETFERT", &x, &y, &wd, &ht, &color);
  GSwclearrect(x, y, wd, ht);
  GSsetfont (LARGE);
  GSsetcolor(color);

  /* Don't know if this value was changed elsewhere... */
  if (!redraw) /* tEST */
  {
    if (sptr->tfr_targ_year[sptr->stepcnt] == sptr->year)
    {
      sptr->tfr_targ_val[sptr->stepcnt] =
        get_TFR(sptr->birthrate[sptr->stepcnt]);
    }
    syptr->tfr_targ_year[sptr->stepcnt] = sptr->tfr_targ_year[sptr->stepcnt];
  }
  GSwwritereal(3, 1, x, y+LARGE,
               (double)sptr->tfr_targ_val[sptr->stepcnt]/LETFRRES);

  GSgetfieldrect("CURRFERT", &x, &y, &wd, &ht, &color);
  GSwclearrect(x, y, wd, ht);
  GSsetfont (LARGE);
  GSsetcolor(color);
/*  GSwwritereal(3, 1, x, y+LARGE,
               (double)sptr->tfr_targ_val[sptr->stepcnt]/LETFRRES);*/
  GSwwritereal(3, 1, x, y+LARGE,
               (double)get_TFR(sptr->birthrate[sptr->stepcnt])/LETFRRES);

  GSgetfieldrect("SETFERTYEAR", &x, &y, &wd, &ht, &color);
  GSwclearrect(x, y, wd, ht);
  GSsetfont(LARGE);
  GSsetcolor(color);
/*  GSwwriteint(4, x, y+LARGE, sptr->tfr_targ_year[sptr->stepcnt]);*/
  GSwwriteint(4, x, y+LARGE, syptr->tfr_targ_year[sptr->stepcnt]);

  GSgetfieldrect("CURRFERTYEAR", &x, &y, &wd, &ht, &color);
  GSwclearrect(x, y, wd, ht);
  GSsetfont(LARGE);
  GSsetcolor(color);
  GSwwriteint(4, x, y+LARGE, sptr->tfr_targ_year[sptr->stepcnt]);
}

/*-------------------------------------------------------------------------------
  Draw Set Life Expectancy box.
-------------------------------------------------------------------------------*/
void display_lifeex_win(boolean redraw)
{
  int x, y, wd, ht;
  GSColor color;
  IPStatetype *sptr;
  IPsaveyear_t *syptr;

  sptr = IPState[Cdisp][Display[Cdisp].curr_sim];
  syptr = IPsaveyear[Cdisp][Display[Cdisp].curr_sim];

  if (redraw)
    GSredrawnamedwindow("setlife");

  GSgetfieldrect("SETLIFE", &x, &y, &wd, &ht, &color);
  GSwclearrect(x, y, wd, ht);
  GSsetfont(LARGE);
  GSsetcolor(color);

  /* Don't know if this value was changed elsewhere... */
  if (!redraw) /* tEST */
  { /*-- if redraw, then we're just restoring what was there, so don't reset --*/
    if (sptr->lifex_targ_year[sptr->stepcnt] == sptr->year)
    {
      sptr->lifex_targ_val[sptr->stepcnt] =
        get_life_expect(sptr->deathrate[sptr->stepcnt]);
    }
    syptr->lifex_targ_year[sptr->stepcnt] =
      sptr->lifex_targ_year[sptr->stepcnt];
  }
  GSwwritereal(3, 1, x, y+LARGE,
               (double)sptr->lifex_targ_val[sptr->stepcnt]/LETFRRES);

  GSgetfieldrect("CURRLIFE", &x, &y, &wd, &ht, &color);
  GSwclearrect(x, y, wd, ht);
  GSsetfont(LARGE);
  GSsetcolor(color);
/*  GSwwritereal(3, 1, x, y+LARGE,
	       (double)sptr->lifex_targ_val[sptr->stepcnt]/LETFRRES);*/ /*tEST*/
  GSwwritereal(3, 1, x, y+LARGE,
	       (double)get_life_expect(sptr->deathrate[sptr->stepcnt])/LETFRRES);

  GSgetfieldrect("SETLIFEYEAR", &x, &y, &wd, &ht, &color);
  GSwclearrect(x, y, wd, ht);
  GSsetfont(LARGE);
  GSsetcolor(color);
/*  GSwwriteint(4, x, y+LARGE, sptr->lifex_targ_year[sptr->stepcnt]);*/
  GSwwriteint(4, x, y+LARGE, syptr->lifex_targ_year[sptr->stepcnt]);

  GSgetfieldrect("CURRLIFEYEAR", &x, &y, &wd, &ht, &color);
  GSwclearrect(x, y, wd, ht);
  GSsetfont(LARGE);
  GSsetcolor(color);
  GSwwriteint(4, x, y+LARGE, sptr->lifex_targ_year[sptr->stepcnt]);
}


/* Draw set migration box */
void display_mig_win(boolean redraw)
{
  int x, y, wd, ht;
  GSColor color;
  IPStatetype *sptr;
  IPsaveyear_t *syptr;

  sptr = IPState[Cdisp][Display[Cdisp].curr_sim];
  syptr = IPsaveyear[Cdisp][Display[Cdisp].curr_sim];

  if (redraw)
    GSredrawnamedwindow("setmig");

  GSgetfieldrect("SETMIG", &x, &y, &wd, &ht, &color);
  GSwclearrect(x, y, wd, ht);
  GSsetfont(LARGE);
  GSsetcolor(color);

  /* Don't know if this value was changed elsewhere... */

  if (!redraw) /* tEST */
  {
    if (sptr->mig_targ_year[sptr->stepcnt] == sptr->year)
    {
      sptr->mig_targ_val[sptr->stepcnt] = sptr->mig[sptr->stepcnt];
    }
    syptr->mig_targ_year[sptr->stepcnt] = sptr->mig_targ_year[sptr->stepcnt];
  }

  GSwwriteint(10, x, y+LARGE, sptr->mig_targ_val[sptr->stepcnt] *
                                        Country[Cdisp].scale_factor);
  GSgetfieldrect("CURRMIG", &x, &y, &wd, &ht, &color);
  GSwclearrect(x, y, wd, ht);
  GSsetfont(LARGE);
  GSsetcolor(color);
/*  GSwwriteint(10, x, y+LARGE, sptr->mig_targ_val[sptr->stepcnt] *
		Country[Cdisp].scale_factor);*//*tEST*/
  GSwwriteint(10, x, y+LARGE, sptr->mig[sptr->stepcnt] *
              Country[Cdisp].scale_factor);/*tEST*/

  GSgetfieldrect("SETMIGYEAR", &x, &y, &wd, &ht, &color);
  GSwclearrect(x, y, wd, ht);
  GSsetfont(LARGE);
  GSsetcolor(color);
/*  GSwwriteint(4, x, y+LARGE, sptr->mig_targ_year[sptr->stepcnt]);*/
  GSwwriteint(4, x, y+LARGE, syptr->mig_targ_year[sptr->stepcnt]);
  GSgetfieldrect("CURRMIGYEAR", &x, &y, &wd, &ht, &color);
  GSwclearrect(x, y, wd, ht);
  GSsetfont(LARGE);
  GSsetcolor(color);
  GSwwriteint(4, x, y+LARGE, sptr->mig_targ_year[sptr->stepcnt]);
}

