/***********************************************************************
International Population module interface functions initiated by the
GeoSim interface handler.
***********************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include "ipincl.h"

#define MODIFYBREAK 14  /* Distinguish between - and + modify rate buttons */

static boolean          FinalExit = FALSE;  /*--- flag for done button of help

                                                 window ---*/
#if 0
extern ButtonStatusType UpArrowStatus;
extern ButtonStatusType DnArrowStatus;
#endif

static int              Oldcountry_code; /*--- Used for picking country and
                                               regions ---*/

/*-------------------------------------------------------------------------------
  Here are the interface functions.
-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------
  Reset for an alternate simulation
-------------------------------------------------------------------------------*/
boolean alternatesim (
  void
)
{
  return(startsim(1990));
}

/*-------------------------------------------------------------------------------
  Toggle birth/death rate buttons with fertility/life expect buttons
-------------------------------------------------------------------------------*/
boolean birthdeathrate (
  void
)
{
  Display[Cdisp].modify_flag = !Display[Cdisp].modify_flag;
  if (!Display[Cdisp].modify_flag)
  {
    GSenablebutton(TFRname[Cdisp], "setfert", DRAW);
    GSenablebutton(Lifexname[Cdisp], "setlife", DRAW);
    GSenablebutton (Displayname[Cdisp], "pyrgraph", NO_DRAW);
    GSsetdeactivenamedwindow(Birthwinname[Cdisp]);
    GSsetdeactivenamedwindow(Deathwinname[Cdisp]);
    GSsetdeactivenamedwindow(Infantwinname[Cdisp]);
  }
  else
  {
    GSdisablebutton(TFRname[Cdisp], "setfert", DRAW);
    GSdisablebutton (Lifexname[Cdisp], "setlife", DRAW);
    GSsetactivenamedwindow(Birthwinname[Cdisp]);
    GSsetactivenamedwindow(Deathwinname[Cdisp]);
    GSsetactivenamedwindow(Infantwinname[Cdisp]);
    GSredrawnamedwindow (Birthwinname[Cdisp]);
    GSredrawnamedwindow (Deathwinname[Cdisp]);
    GSredrawnamedwindow (Infantwinname[Cdisp]);
  }
  update_display(Cdisp);
  if (!InWalkThru)
    GSdisplayhelpmessage(4);
  return(TRUE);
}


/*-------------------------------------------------------------------------------
  Initiate "Controls" help screen
-------------------------------------------------------------------------------*/
boolean controlscmd (
  void
)
{
  boolean return_value;

  HelpWinActive = TRUE;
  sprintf(Helpfile, "%s%s", Infopath, CONTROLS_HLP);
  return_value = GSdohelpscreens(Helpfile, FirstPage, Use_bstore, 10);
  if (MapSelect)
  {
#if 0
    drawuparrow(DISABLED, FALSE);
    drawdownarrow(DISABLED, FALSE);
#endif
  }
  return (return_value);
}

/*-------------------------------------------------------------------------------
  Decrement target total fertility rate value
-------------------------------------------------------------------------------*/
boolean decfert (
  void
)
{
  IPStatetype *sptr = IPState[Cdisp][Display[Cdisp].curr_sim];

if (sptr->tfr_targ_val[sptr->stepcnt] >= 990)
  GSenablebutton("setfert", "incfert", DRAW);
  if (changefert(-1))
  {
    if (sptr->tfr_targ_val[sptr->stepcnt] <= 50)
    {
      GSdisablebutton("setfert", "decfert", DRAW);
      Drag_Used = TRUE; /* walkthru checks this flag */
    }
    return TRUE;
  }
  else
    return FALSE;
}

/*-------------------------------------------------------------------------------
  Decrement target total fertility year value by 5 if it is still
  greater than the current simulation year
-------------------------------------------------------------------------------*/
boolean decfyear (
  void
)
{
/*  IPStatetype *sptr = IPState[Cdisp][Display[Cdisp].curr_sim];

  GSenablebutton("setfert", "incfyear", DRAW);
  if (changeyear(-5, sptr, &sptr->tfr_targ_year[sptr->stepcnt], "SETFERTYEAR"))
  {
    if (sptr->tfr_targ_year[sptr->stepcnt] <= sptr->year)
      GSdisablebutton("setfert", "decfyear", DRAW);
    return TRUE;
  }
  else
    return(FALSE);*/

  IPStatetype *sptr = IPState[Cdisp][Display[Cdisp].curr_sim];
  IPsaveyear_t *syptr = IPsaveyear[Cdisp][Display[Cdisp].curr_sim];

  GSenablebutton("setfert", "incfyear", DRAW);
  if (changeyear(-5, sptr, &syptr->tfr_targ_year[sptr->stepcnt],
                 "SETFERTYEAR"))
  {
    if (syptr->tfr_targ_year[sptr->stepcnt] <= sptr->year)
      GSdisablebutton("setfert", "decfyear", DRAW);
    return TRUE;
  }
  else
    return(FALSE);}

/*------------------------------------------------------------------------------
  Decrement the life expectancy value by .2 if it is greater than 20.0.
------------------------------------------------------------------------------*/
boolean declifeexp (
  void
)
{
  IPStatetype *sptr = IPState[Cdisp][Display[Cdisp].curr_sim];

if (sptr->lifex_targ_val[sptr->stepcnt] >= 9990)
  GSenablebutton("setlife", "inclifeexp", DRAW);
  if (changelifex(-2))
  {
    if (sptr->lifex_targ_val[sptr->stepcnt] <= 2000)
      GSdisablebutton("setlife", "declifeexp", DRAW);
    return TRUE;
  }
  else
    return FALSE;
}

/*------------------------------------------------------------------------------
  Decrement target life expectancy year by 5 if it is greater than
  the current simulation year
------------------------------------------------------------------------------*/
boolean declyear (
  void
)
{
/*  IPStatetype *sptr = IPState[Cdisp][Display[Cdisp].curr_sim];

  GSenablebutton("setlife", "inclyear", DRAW);
  if (changeyear(-5, sptr, &sptr->lifex_targ_year[sptr->stepcnt], "SETLIFEYEAR"))
  {
    if (sptr->lifex_targ_year[sptr->stepcnt] <= sptr->year)
      GSdisablebutton("setlife", "declyear", DRAW);
    return TRUE;
  }
  else
    return(FALSE);*/

  IPStatetype *sptr = IPState[Cdisp][Display[Cdisp].curr_sim];
  IPsaveyear_t *syptr = IPsaveyear[Cdisp][Display[Cdisp].curr_sim];

  GSenablebutton("setlife", "inclyear", DRAW);
  if (changeyear(-5, sptr, &syptr->lifex_targ_year[sptr->stepcnt],
                 "SETLIFEYEAR"))
  {
    if (syptr->lifex_targ_year[sptr->stepcnt] <= sptr->year)
      GSdisablebutton("setlife", "declyear", DRAW);
    return TRUE;
  }
  else
    return(FALSE);
}

/*-------------------------------------------------------------------------------
  Decrement migration rate
-------------------------------------------------------------------------------*/
boolean decmig (
  void
)
{
  IPStatetype *sptr = IPState[Cdisp][Display[Cdisp].curr_sim];

if (sptr->mig_targ_val[sptr->stepcnt] >= 100000L)
  GSenablebutton("setmig", "incmig", DRAW);

  if (changemig(-1))
  {
    if (sptr->mig_targ_val[sptr->stepcnt] <= -100000L)
      GSdisablebutton("setmig", "decmig", DRAW);
    return TRUE;
  }
  else
    return FALSE;
}

/*-------------------------------------------------------------------------------
  Decrement target migration year by 5 if it is greater than
  the current simulation year
-------------------------------------------------------------------------------*/
boolean decmyear (
  void
)
{
/*  IPStatetype *sptr = IPState[Cdisp][Display[Cdisp].curr_sim];

  GSenablebutton("setmig", "incmyear", DRAW);
  if (changeyear(-5, sptr, &sptr->mig_targ_year[sptr->stepcnt], "SETMIGYEAR"))
  {
    if (sptr->mig_targ_year[sptr->stepcnt] <= sptr->year)
      GSdisablebutton("setmig", "decmyear", DRAW);
    return TRUE;
  }
  else
    return(FALSE);*/

  IPStatetype *sptr = IPState[Cdisp][Display[Cdisp].curr_sim];
  IPsaveyear_t *syptr = IPsaveyear[Cdisp][Display[Cdisp].curr_sim];

  GSenablebutton("setmig", "incmyear", DRAW);
  if (changeyear(-5, sptr, &syptr->mig_targ_year[sptr->stepcnt],
                 "SETMIGYEAR"))
  {
    if (syptr->mig_targ_year[sptr->stepcnt] <= sptr->year)
      GSdisablebutton("setmig", "decmyear", DRAW);
    return TRUE;
  }
  else
    return(FALSE);
}

/*-------------------------------------------------------------------------------
  Display the version number and then the introductory help screen
-------------------------------------------------------------------------------*/
boolean do_intro (
  void
)
{
  HelpWinActive = TRUE;
  GSdeactivatefunction ("do_intro");
  GSdisplayversion(VersionFile);

/*  GSdisplayhelpmessage(ParmSavemsg = 6);*/
  ParmSavemsg = 6;
  if (!StartWalkThru)
  {
    QuickStartSavemsg = 6;
    dointroscreen();
  }

  HelpWinActive = TRUE;
  sprintf(Helpfile, "%s%s", Infopath, INTRO_HLP);
  return(GSdohelpscreens(Helpfile, FirstPage, Use_bstore, 8));
}

/*-------------------------------------------------------------------------------
  Finished looking at a help file
-------------------------------------------------------------------------------*/
boolean done (
  void
)
{
  static boolean firsttime = TRUE;

  HelpWinActive = FALSE;
  if (FinalExit)	/* if the exit flag is set, then quit Intlpop */
    GSquit(NULL);

  GSremovenamedwindow("help");
  if (firsttime)
  {
    firsttime = FALSE;
    GSdisplayhelpmessage(13);
    if (StartWalkThru)
    {
      StartWalkThru = FALSE;
      GSdisplayhelpmessage(6);
      walkthruinit();
    }
    else if (!Use_bstore)
    {
      GSredrawnamedwindow("instruction");
      drawintroscreen();
    }
  }
  else
  {
    GSdisplayhelpmessage(-2);
    if (MapSelect)
    {
    
    /*  drawuparrow(UpArrowStatus, FALSE);
      drawdownarrow(DnArrowStatus, FALSE);*/
    }
    if (!Use_bstore) redraw_all();
  }
  return(TRUE);
}


/*-------------------------------------------------------------------------------
  Quit button.  Show the final message screen.
-------------------------------------------------------------------------------*/
boolean doquit (
  void
)
{
  GSremovenamedwindow("quit");
  if (!Use_bstore) redraw_all();
  GSdisplayhelpmessage (-2);
  HelpWinActive = TRUE;
  FinalExit = TRUE;
  GSdisplayhelpmessage(16);
  sprintf(Helpfile, "%s%s", Infopath, EXIT_HLP);
  return(GSdohelpscreens(Helpfile, FirstPage, Use_bstore, 16));
}

/*-------------------------------------------------------------------------------
  
-------------------------------------------------------------------------------*/
boolean dragbars (
  int cx,
  int cy,
  DragStatusType status
)
{
  static boolean      male_flag;  /* TRUE iff moving male bar */
  static int          bar;
  static long         oldwd;
  static long         initbinpop1;
  static long         initbinpop2;
  static long         restofpop;
         long         scale;
         long         safescale;
  static int          prevnewwd;  /* previous value for newwd */
  static long        *popptr;
         int          x, y, ht, wd, bh;
         GSColor      color;
         GSColor      barcolor;
         int          newwd;
         int          barx;
         int          i;
         long         pop;  /* new bar population value */
         IPStatetype *sptr;
         int          stepcnt;
         ControlType  control;

  GSgetlastcontrol(&control);
  if (status == DRAG_INIT)
  {
    GSgetfieldrect("BARS", &x, &y, &wd, &bh, &color);
    GSgetfieldrect("PYRBUTTON", &x, &y, &wd, &ht, &color);
    if ((Cdisp = checkwindow(control.gname)) < 0) return(FALSE);
      sptr = IPState[Cdisp][Display[Cdisp].curr_sim];
    stepcnt = sptr->stepcnt;
    prevnewwd = -1;
    /* Don't let the bar change even if the mouse slips */
    bar = (ht - cy) / (bh-1);
    if (cx > wd/2)
    {
      male_flag = FALSE;
      popptr = Display[Cdisp].female;
    }
    else
    {
      male_flag = TRUE;
      popptr = Display[Cdisp].male;
    }
    initbinpop1 = popptr[2*bar];
    if (bar != 8)
      initbinpop2 = popptr[2*bar+1];
    else
      initbinpop2 = 0;
    if (sptr->totpop[stepcnt] > 50000000L)
    {
      safescale = 1000;
      scale = IDIV(sptr->totpop[stepcnt], 1000) * Display[Cdisp].bound;
    }
    else
    {
      safescale = 1;
      scale = sptr->totpop[stepcnt] * Display[Cdisp].bound;
    }
    if (sptr->totpop[stepcnt] != 0)
    {
      oldwd = (int)RESCALE((initbinpop1 + initbinpop2), (wd/2) * 100, scale);
      oldwd /= safescale;
    }
    else
      oldwd = 0;

    if (male_flag)
    {
      if (oldwd > 2)
        GSwclearrect(x+wd/2-oldwd, y+ht-(bh-1)*(bar+1), oldwd, bh);
      GSsetcolor(color);
      GSwdrawrect(x+wd/2-oldwd-1, y-1+ht-(bh-1)*(bar+1), oldwd, bh);
      popptr = sptr->male_pop[stepcnt];
    }
    else
    {
      if (oldwd > 2)
        GSwclearrect(x+wd/2, y+ht-(bh-1)*(bar+1), oldwd, bh);
      GSsetcolor(color);
      GSwdrawrect(x+wd/2-1, y-1+ht-(bh-1)*(bar+1), oldwd, bh);
      popptr = sptr->fem_pop[stepcnt];
    }
    for (restofpop=0, i=0; i<17; i++)
      restofpop += sptr->male_pop[stepcnt][i] + sptr->fem_pop[stepcnt][i];
    if (bar != 8)
      restofpop -= (popptr[2*bar] + popptr[2*bar+1]);
    else
      restofpop -= popptr[2*bar];
  }
  else
  if (status == DRAG_PROCESS)
  {
    GSgetfieldrect("BARS", &x, &y, &wd, &bh, &color);
    GSgetfieldrect("PYRBUTTON", &x, &y, &wd, &ht, &color);
    if (male_flag)
    {
      barx = MIN(cx, wd/2);
      barx = MAX(barx, wd/2-oldwd);
      newwd = wd/2 - barx;
    }
    else
    {
      barx = MIN(cx, wd/2+oldwd);
      barx = MAX(barx, wd/2);
      newwd = barx - wd/2;
    }
    pop = popptr[2*bar] = RESCALE(initbinpop1, newwd, oldwd);
    if (bar != 8)
      pop += popptr[2*bar+1] = RESCALE(initbinpop2, newwd, oldwd);

    if (newwd != prevnewwd)
    {
      /* Redraw the bar */
      barcolor = GSgetlistcolor (1, bar % 4);
      if (male_flag)
      {
        if (oldwd > 2)
          GSwclearrect(x+wd/2-oldwd-1, y-1+ht-(bh-1)*(bar+1), oldwd, bh);
        if (newwd > 2)
        {
          GSsetcolor(barcolor);
          GSwfillrect(x+wd/2-newwd, y+ht-(bh-1)*(bar+1), newwd-2, bh-2);
        }
      }
      else
      {
        if (oldwd > 2)
          GSwclearrect(x+wd/2-1, y-1+ht-(bh-1)*(bar+1), oldwd, bh);
        if (newwd > 2)
        {
          GSsetcolor(barcolor);
          GSwfillrect(x+wd/2, y+ht-(bh-1)*(bar+1), newwd-2, bh-2);
        }
      }
      GSgetfieldrect("SIMPOP", &x, &y, &wd, &ht, &color);
      GSwclearrect(x, y, wd, ht);
      color = GSgetlistcolor(0,Display[Cdisp].curr_sim);
      GSsetfont(SMALL);
      GSsetcolor(color);
      GSwwritecommanum(x, y+SMALL,
                (double)(restofpop + pop)*(double)Country[Cdisp].scale_factor);
    }
    prevnewwd = newwd;
  }
  else
  if (status == DRAG_FINAL)
  {
    display_bar_graph(Cdisp);
    Drag_Used = TRUE; /* checked by walk-through tutor */
  }
  return(TRUE);
}


/*-------------------------------------------------------------------------------
  Helper function to do the work for draglargemap and dragsmallmap
-------------------------------------------------------------------------------*/
boolean dragmaphelp (
  int       cx,
  int       cy,
  DragStatusType       status,
  GSPicture *pic)
{
  int       x, y, wd, ht;
  int       lx, ly, lwd, lht;
  GSColor   color;
  unsigned  short country_code;
  BigString tmpstr;

  GSsetactivenamedwindow ("map");

  if (status == DRAG_INIT || status == DRAG_PROCESS)
  {
    if (pic == &Regionpic)
    { /* Clear the old xor box and write the new */
      GSgetfieldrect("LMAP", &lx, &ly, &lwd, &lht, &color);
      GSgetfieldrect("SMAP", &x, &y, &wd, &ht, &color);
      GSdrawxorbox(Xorx+x, Xory+y, Xorwd, Xorht);
      compute_xorbox(cx, cy, MapScale, wd, ht, lwd, lht);
      GSdrawxorbox(Xorx+x, Xory+y, Xorwd, Xorht);
    }
  }
  else if (status == DRAG_FINAL)
  {
    if (pic == &Regionpic)
    { /*  redraw large map here  */
      GSgetfieldrect("SMAP", &x, &y, &wd, &ht, &color);
      GSgetfieldrect("LMAP", &lx, &ly, &lwd, &lht, &color);
      GSdrawxorbox(Xorx+x, Xory+y, Xorwd, Xorht);
      compute_xorbox(cx, cy, MapScale, wd, ht, lwd, lht);
      GSdrawxorbox(Xorx+x, Xory+y, Xorwd, Xorht);
      GSsetcolor(color);
      GSwfillrect(lx, ly, lwd, lht);
      GSwdrawpict(&Countrypic, lx, ly, lwd, lht, 2*Xorx, 2*Xory, MapScale,
                  code2color);
      return (TRUE);
    }
  }
  else if (status == MOUSE_CLICK)
  {
    if (pic == &Countrypic)
    { /* Adjust coordinate in large map */
      cx = Xorx*2 + cx/MapScale;
      cy = Xory*2 + cy/MapScale;
    }
    GSgetpixel(pic, cx, cy, &country_code, 1);
    if (country_code < Realnumregions+Realnumcountries)
    {
      Pickno = country_code;
      GSgetfieldrect("SNAMEBACK", &x, &y, &wd, &ht, &color);
      GSsetcolor(color);
      GSwfillrect (x, y, wd, ht);
      GSgetfieldrect("SNAME", &x, &y, &wd, &ht, &color);
      GSsetfont (LARGE);
      GSsetcolor(color);
      GSwwritemsg (x, y+LARGE,  Country_list[country_code].name);
      return(TRUE);
    }
    else
      return (FALSE);  /* no country found for map code  */
  }
  else if (status == MOUSE_INSIDE)
  {
    if (pic == &Countrypic)
    { /* Adjust coordinate in large map */
      cx = Xorx*2 + cx/MapScale;
      cy = Xory*2 + cy/MapScale;
    }
    GSgetpixel(pic, cx, cy, &country_code, 1);
    if (Oldcountry_code != country_code)
    { /* Put up the name of the new country */
      Oldcountry_code = country_code;
      GSgetfieldrect("CNAMEBACK", &x, &y, &wd, &ht, &color);
      GSsetcolor(color);
      GSwfillrect (x, y, wd, ht);
      GSgetfieldrect("CNAME", &x, &y, &wd, &ht, &color);
      if (country_code < Realnumregions+Realnumcountries)
      {
        sprintf(tmpstr, "%s", Country_list[country_code].name);
        GSsetfont (LARGE);
        GSsetcolor(color);
        GSwwritemsg (x, y+LARGE, tmpstr);
      }
      else
      { /* Put up no country/region message */
        GSsetfont(LARGE);
        GSsetcolor(color);
        GSwwritenamedmsg(x, y+LARGE, "ctynone");
      }
    }
  }
  else if (status == MOUSE_OUTSIDE)
  { /* Put up no country/region message */
    Oldcountry_code = -1;
    GSgetfieldrect("CNAMEBACK", &x, &y, &wd, &ht, &color);
    GSsetcolor(color);
    GSwfillrect (x, y, wd, ht);
    GSgetfieldrect("CNAME", &x, &y, &wd, &ht, &color);
    GSsetfont(LARGE);
    GSsetcolor(color);
    GSwwritenamedmsg(x, y+LARGE, "ctynone");
  }
  return(TRUE);
}


/*-------------------------------------------------------------------------------
  When moving/dragging inside the country (large) map
-------------------------------------------------------------------------------*/
boolean draglargemap(int cx, int cy, DragStatusType status)
{
  if (status == DRAG_FINAL)
    status = MOUSE_CLICK;
  return(dragmaphelp(cx, cy, status, &Countrypic));
}


/*-------------------------------------------------------------------------------
  When dragging inside the region (small) map
-------------------------------------------------------------------------------*/
boolean dragsmallmap(int cx, int cy, DragStatusType status)
{
  if (status == DRAG_FINAL)
    Drag_Used = TRUE; /* walkthru checks this flag */
  return(dragmaphelp(cx, cy, status, &Regionpic));
}


/*-------------------------------------------------------------------------------
  Initiate the "expected" scenario
-------------------------------------------------------------------------------*/
boolean expectedsim(void)
{
  return(startsim(2000));
}


/*-------------------------------------------------------------------------------
 Display the "GeoSim Help" file
-------------------------------------------------------------------------------*/
boolean geocmd(void)
{
  boolean return_value;

  HelpWinActive = TRUE;
  sprintf(Helpfile, "%s%s", Infopath, GEO_HLP);
  return_value = GSdohelpscreens(Helpfile, FirstPage, Use_bstore, 10);
  if (MapSelect)
  {
#if 0
    drawuparrow(DISABLED, FALSE);
    drawdownarrow(DISABLED, FALSE);
#endif
  }
  return (return_value);
}

/*-------------------------------------------------------------------------------
  "Prev" button on a help screen
-------------------------------------------------------------------------------*/
boolean helpprev(void)
{
  return(GSdohelpscreens(Helpfile, PrevPage, Use_bstore, CURR_MSG));
}


/* "next" button on a help screen */
boolean helpnext(void)
{
  return(GSdohelpscreens(Helpfile, NextPage, Use_bstore, CURR_MSG));
}


/*-------------------------------------------------------------------------------
  Increase the fertility rate.
-------------------------------------------------------------------------------*/
boolean incfert(void)
{
  IPStatetype *sptr = IPState[Cdisp][Display[Cdisp].curr_sim];

  if (sptr->tfr_targ_val[sptr->stepcnt] <= 50)
    GSenablebutton("setfert", "decfert", DRAW);
  if (changefert(1))
  {
    if (sptr->tfr_targ_val[sptr->stepcnt] >= 990)
      GSdisablebutton("setfert", "incfert", DRAW);
    return TRUE;
  }
  else
    return FALSE;
}


/*-------------------------------------------------------------------------------
  Increase target year for TFR change by 5 if it is less than 2050
-------------------------------------------------------------------------------*/
boolean incfyear(void)
{
/*  IPStatetype *sptr = IPState[Cdisp][Display[Cdisp].curr_sim];

  GSenablebutton("setfert", "decfyear", DRAW);
  if (InWalkThru)
    if (sptr->tfr_targ_year[sptr->stepcnt] >= 2005)
    {
      Drag_Used = TRUE; / walkthru checks this flag /
      GSdisablebutton("setfert", "incfyear", DRAW);
    }
  if (changeyear(5, sptr, &sptr->tfr_targ_year[sptr->stepcnt], "SETFERTYEAR"))
  {
    if (sptr->tfr_targ_year[sptr->stepcnt] >= 2050)
      GSdisablebutton("setfert", "incfyear", DRAW);
    return TRUE;
  }
  else
    return FALSE; */

  IPStatetype *sptr = IPState[Cdisp][Display[Cdisp].curr_sim];
  IPsaveyear_t *syptr = IPsaveyear[Cdisp][Display[Cdisp].curr_sim];

  GSenablebutton("setfert", "decfyear", DRAW);
  if (InWalkThru)
    if (syptr->tfr_targ_year[sptr->stepcnt] >= 2005)
    {
	  Drag_Used = TRUE;			/* walkthru checks this flag */
      GSdisablebutton("setfert", "incfyear", DRAW);
    }
  if (changeyear(5, sptr, &syptr->tfr_targ_year[sptr->stepcnt],
      "SETFERTYEAR"))
  {
    if (syptr->tfr_targ_year[sptr->stepcnt] >= 2050)
      GSdisablebutton("setfert", "incfyear", DRAW);
    return TRUE;
  }
  else
    return FALSE;
}


/*-------------------------------------------------------------------------------
  Increment the life expectancy value by 0.2 if it is less than 99.9
-------------------------------------------------------------------------------*/
boolean inclifeexp(void)
{
  IPStatetype *sptr = IPState[Cdisp][Display[Cdisp].curr_sim];

if (sptr->lifex_targ_val[sptr->stepcnt] <= 2000)
  GSenablebutton("setlife", "declifeexp", DRAW);
  if (changelifex(2))
  {
    if (sptr->lifex_targ_val[sptr->stepcnt] >= 9990)
      GSdisablebutton("setlife", "inclifeexp", DRAW);
    return TRUE;
  }
  else
    return FALSE;
}


/*-------------------------------------------------------------------------------
  Increment the year for target life expectancy change by 5 if it is
  still less than 2050
-------------------------------------------------------------------------------*/
boolean inclyear (
  void
)
{
/*  IPStatetype *sptr = IPState[Cdisp][Display[Cdisp].curr_sim];
  GSenablebutton("setlife", "declyear", DRAW);
  if (changeyear(5, sptr, &sptr->lifex_targ_year[sptr->stepcnt], "SETLIFEYEAR"))
  {
    if (sptr->lifex_targ_year[sptr->stepcnt] >= 2050)
      GSdisablebutton("setlife", "inclyear", DRAW);
    return TRUE;
  }
  else
    return FALSE;*/

  IPStatetype *sptr = IPState[Cdisp][Display[Cdisp].curr_sim];
  IPsaveyear_t *syptr = IPsaveyear[Cdisp][Display[Cdisp].curr_sim];

  GSenablebutton("setlife", "declyear", DRAW);
  if (changeyear(5, sptr, &syptr->lifex_targ_year[sptr->stepcnt],
                 "SETLIFEYEAR"))
  {
    if (syptr->lifex_targ_year[sptr->stepcnt] >= 2050)
      GSdisablebutton("setlife", "inclyear", DRAW);
    return TRUE;
  }
  else
    return FALSE;
}

/*-------------------------------------------------------------------------------
  Increment migration value
-------------------------------------------------------------------------------*/
boolean incmig(void)
{
  IPStatetype *sptr = IPState[Cdisp][Display[Cdisp].curr_sim];

  if (sptr->mig_targ_val[sptr->stepcnt] <= -100000L)
    GSenablebutton("setmig", "decmig", DRAW);

  if (changemig(1))
  {
    if (sptr->mig_targ_val[sptr->stepcnt] >= 100000L)
      GSdisablebutton("setmig", "incmig", DRAW);
    return TRUE;
  }
  else
    return FALSE;
}


/*-------------------------------------------------------------------------------
  Increment the year for target migration change by 5 if it is
  still less than 2050
-------------------------------------------------------------------------------*/
boolean incmyear(void)
{
/*  IPStatetype *sptr = IPState[Cdisp][Display[Cdisp].curr_sim];

  GSenablebutton("setmig", "decmyear", DRAW);
  if (changeyear(5, sptr, &sptr->mig_targ_year[sptr->stepcnt], "SETMIGYEAR"))
  {
    if (sptr->mig_targ_year[sptr->stepcnt] >= 2050)
      GSdisablebutton("setmig", "incmyear", DRAW);
    return TRUE;
  }
  else
    return FALSE;*/

  IPStatetype *sptr = IPState[Cdisp][Display[Cdisp].curr_sim];
  IPsaveyear_t *syptr = IPsaveyear[Cdisp][Display[Cdisp].curr_sim];

  GSenablebutton("setmig", "decmyear", DRAW);
  if (changeyear(5, sptr, &syptr->mig_targ_year[sptr->stepcnt], "SETMIGYEAR"))
  {
    if (syptr->mig_targ_year[sptr->stepcnt] >= 2050)
      GSdisablebutton("setmig", "incmyear", DRAW);
    return TRUE;
  }
  else
    return FALSE;
}


/*-------------------------------------------------------------------------------
  Bring up general help screen
-------------------------------------------------------------------------------*/
boolean introhelpcmd (void)
{
  QuickStartSavemsg = GSgetcurrmsg();
  dointroscreen();
  return(TRUE);
}


/*-------------------------------------------------------------------------------
  Initiate main help window
  Remove instructions window and activate top/bottom windows 
-------------------------------------------------------------------------------*/
boolean instrdone(void)
  {
  QuickStartActive = FALSE;
  GSremovenamedwindow("instruction");
  GSsetdeactivenamedwindow("instruction");
  GSsetactivenamedwindow("root");
  if (!InWalkThru)
    if (TutorEntry)
      GSenablebutton("root", "glossary", NO_DRAW);
    else
      GSenablebutton("root", "file", NO_DRAW);
  GSenablemenuitem("help", "p0");
  GSenablemenuitem("help", "p4");
  GSsetactivenamedwindow("top");
  GSsetactivenamedwindow("bottom");
  GSinvisiblebutton("message", "done", NO_DRAW);
  if (InWalkThru)
    GSenablebutton("root", "qtwalk", NO_DRAW);
  redraw_all ();
  GSdisplayhelpmessage(QuickStartSavemsg);
  return(TRUE);
}

/*-------------------------------------------------------------------------------

-------------------------------------------------------------------------------*/
boolean maincmd(void)
{
  boolean return_value;

  HelpWinActive = TRUE;
  sprintf(Helpfile, "%s%s", Infopath, MAIN_HLP);
  return_value = GSdohelpscreens(Helpfile, FirstPage, Use_bstore, 10);
  if (MapSelect)
  {
#if 0
    drawuparrow(DISABLED, FALSE);
    drawdownarrow(DISABLED, FALSE);
#endif
  }
  return (return_value);
}


/*-------------------------------------------------------------------------------
 'CANCEL' button in pick country map display
-------------------------------------------------------------------------------*/
boolean mapcancel(void)
{
  MapSelect = FALSE;
  GSclearcharfunc();
  Pickno = -1;
  endselectcountry();
  GSremovenamedwindow("map");
  GSsetactivenamedwindow("root");
  GSsetactivenamedwindow("top");
  GSsetactivenamedwindow("bottom");
  GSsetactivenamedwindow("message");
  if (!InWalkThru)
    if (TutorEntry)
      GSenablebutton("root", "glossary", NO_DRAW);
    else
      GSenablebutton("root", "file", NO_DRAW);

  redraw_all();
  GSdisplayhelpmessage(MapSelectSavemsg);
  return(TRUE);
}


/*------------------------------------------------------------------------------
  done with map selection 
------------------------------------------------------------------------------*/
boolean mapdone (void)
{
  MapSelect = FALSE;
  GSclearcharfunc();
  endselectcountry();
  GSremovenamedwindow("map");
  GSsetactivenamedwindow("root");
  GSsetactivenamedwindow("top");
  GSsetactivenamedwindow("bottom");
  GSsetactivenamedwindow("message");
  if (!InWalkThru)
    if (TutorEntry)
      GSenablebutton("root", "glossary", NO_DRAW);
    else
      GSenablebutton("root", "file", NO_DRAW);

  redraw_all();
  if (Pickno != -1)
  {
    if (!readcountry(Pickno))
      return(FALSE);
    init_display_data(Cdisp);
    update_display(Cdisp);
    if (!InWalkThru)
      GSdisplayhelpmessage(ParmSavemsg = 4);
  }
  else
    GSdisplayhelpmessage(MapSelectSavemsg);

  return(TRUE);
}


/*-------------------------------------------------------------------------------
 Modify a cohort birthrate.
-------------------------------------------------------------------------------*/
boolean modifybirthrate(void)
{
  int         cursx, cursy;  /* cursor position in window */
  BigString   tmpstr;
  IPStatetype *sptr;
  long        *female;
  int         stepcnt;
  int         num; /* which rate position */
  int         x, y, ht, wd, bh, dum; /* display rectangle values */
  GSColor     color; /* display color */
  ControlType control;

  GSgetlastcontrol(&control);

/*
  GSgetcurrentwindowname(winame);
  if (strcmp(winame, "topbirth") == 0)
    Cdisp = DISPLAY1;
  else if (strcmp(winame, "bottombirth") == 0)
    Cdisp = DISPLAY2;
*/
  Cdisp = checkwindow(control.gname);
  if (Cdisp == DISPLAY1)
  {
    GSsetcurrentnamedwindow(control.gname);
  }
  else if (Cdisp == DISPLAY2)
  {
    GSsetcurrentnamedwindow(control.gname);
  }
  else
  {
    GSelog(0, "ERROR - in modifybirthrate with window |%s|\n", control.gname);
    return(FALSE);
  }
  GSgetfieldrect ("BARS", &dum, &dum, &dum, &bh, &color);
  sptr = IPState[Cdisp][Display[Cdisp].curr_sim];
  stepcnt = sptr->stepcnt;
  female = sptr->fem_pop[stepcnt];
  GSgetcursorxy(&cursx, &cursy);
  num = 4 - (cursy-3)/(bh-1);

  /*--- Increment or decrement as appropriate. ---*/
  /*--- We actually modify the cohort values, and then reconstruct
        the sum for display purposes. ---*/
  if (cursx > MODIFYBREAK)
  { /*--- increment the cohort values by 1 if still less than MAXRATE
          and TFR still less than 9.9 ---*/
    if (get_TFR(sptr->birthrate[stepcnt]) < MAXTFR)
    {
      if (sptr->birthrate[stepcnt][2*num] < MAXRATE)
      {
        sptr->birthrate[stepcnt][2*num] =
          (IDIV(sptr->birthrate[stepcnt][2*num], RATERES) + 1) * RATERES;
      }
      if (num != 4)
      { /*--- 50-59 cohort shown, but 50-54 actually computed ---*/
        if (sptr->birthrate[stepcnt][2*num+1] < MAXRATE)
        {
          sptr->birthrate[stepcnt][2*num+1] =
            (IDIV(sptr->birthrate[stepcnt][2*num+1], RATERES) + 1) * RATERES;
        }
      }
    } /*--- if (get_TFR(sptr->birthrate[stepcnt]) < MAXTFR) ---*/
  }
  else
  { /*--- decrement the cohort values by 1 if greater than zero ---*/
    if (sptr->birthrate[stepcnt][2*num] >= MAXRATE)
    {
      strcpy(tmpstr, control.control.name);
      tmpstr[strlen(tmpstr)-1] = 'p';
      GSenablebutton(control.gname, tmpstr, DRAW); 
    }
    sptr->birthrate[stepcnt][2*num] =
      (IDIV(sptr->birthrate[stepcnt][2*num], RATERES) - 1) * RATERES;
    if (sptr->birthrate[stepcnt][2*num] < 0)
    {
      sptr->birthrate[stepcnt][2*num] = 0;
/*      GSdisablebutton(control.gname, control.control.name, DRAW);*/
    }
    if (num != 4)
    { /*--- 50-59 cohort shown, but 50-54 actually computed ---*/
      if (sptr->birthrate[stepcnt][2*num] >= MAXRATE)
      {
        strcpy(tmpstr, control.control.name);
        tmpstr[strlen(tmpstr)-1] = 'p';
        GSenablebutton(control.gname, tmpstr, DRAW); 
      }
      sptr->birthrate[stepcnt][2*num+1] =
        (IDIV(sptr->birthrate[stepcnt][2*num+1], RATERES) - 1) * RATERES;
      if (sptr->birthrate[stepcnt][2*num+1] < 0)
      {
        sptr->birthrate[stepcnt][2*num+1] = 0;
/*        GSdisablebutton(control.gname, control.control.name, DRAW);*/
      }
    }
  } /* else decrement */

  /*--- Display the new rate value ---*/
  GSsetcurrentnamedwindow(Displayname[Cdisp]);
  GSgetfieldrect("BIRTHRATE", &x, &y, &wd, &ht, &color);
  GSwclearrect(x, y + 1 + (4-num)*(bh-1), wd, SMALL+1);
  GSsetfont(SMALL);
  GSsetcolor(color);
  GSwwriteint(3, x, y + SMALL + (4-num)*(bh-1),
              rebuild_birthrate(num, sptr, female, stepcnt));

  /*--- Display current TFR ---*/
  GSsetcurrentnamedwindow(TFRname[Cdisp]);
  GSgetfieldrect("TFR", &x, &y, &wd, &ht, &color);
  GSwclearrect(x, y, wd, ht);
  GSsetfont (SMALL);
  GSsetcolor(color);
  sptr->tfr_targ_val[sptr->stepcnt] = get_TFR(sptr->birthrate[stepcnt]);
  GSwwritereal(3, 1, 2, y-1+SMALL,
	       (double)sptr->tfr_targ_val[sptr->stepcnt] / LETFRRES);
  GSclick();
  return(TRUE);
}


/*------------------------------------------------------------------------------
  Modify a cohort deathrate 
------------------------------------------------------------------------------*/
boolean modifydeathrate(void)
{
  int              cursx, cursy;  /* cursor position in window */
  IPStatetype      *sptr;
  long             *male, *female;
  int              stepcnt;
  long             sum;
/*------------------------------------------------------------------
  long             old_deathrate_1;  USED IN MANAGING BUTTONS
  long             old_deathrate_2;  NOT USED RIGHT NOW
------------------------------------------------------------------*/
  int              num; /* which rate position */
  int              x, y, ht, wd, bh, dum; /* display rectangle values */
  GSColor          color; /* display color */
  ControlType      control;

  GSgetlastcontrol(&control);

  Cdisp = checkwindow(control.gname);
  if (Cdisp == DISPLAY1)
  {
    GSsetcurrentnamedwindow("topdeath");
  }
  else if (Cdisp == DISPLAY2)
  {
    GSsetcurrentnamedwindow("bottomdeath");
  }
  else
  {
    GSelog(0, "ERROR - in modifydeathrate with window |%s|\n", control.gname);
    return(FALSE);
  }

  GSgetfieldrect ("BARS", &dum, &dum, &dum, &bh, &color);
  sptr = IPState[Cdisp][Display[Cdisp].curr_sim];
  stepcnt = sptr->stepcnt;
  male = sptr->male_pop[stepcnt];
  female = sptr->fem_pop[stepcnt];
  GSgetcursorxy(&cursx, &cursy);
  num = 8 - (cursy-3)/(bh-1);

  /*--- Increment or decrement as appropriate. ---*/
  /*--- We actually modify the cohort values, and then reconstruct
        the sum for display purposes. ---*/
/*------------------------------------------------------------------
  NOT USED RIGHT NOW (used for button management)
  old_deathrate_1 = sptr->deathrate[stepcnt][2*num+1];
  old_deathrate_2 = sptr->deathrate[stepcnt][2*num+2];
------------------------------------------------------------------*/
  if (cursx > MODIFYBREAK)
  { /* increment the cohort values by 1 if still less than MAXRATE */
    if (sptr->deathrate[stepcnt][2*num+1] < MAXRATE)
      sptr->deathrate[stepcnt][2*num+1] =
        (IDIV(sptr->deathrate[stepcnt][2*num+1], RATERES) + 1) * RATERES;
    if (num != 8)
    { /* 80+ is only one cohort */
      if (sptr->deathrate[stepcnt][2*num+2] < MAXRATE)
        sptr->deathrate[stepcnt][2*num+2] =
          (IDIV(sptr->deathrate[stepcnt][2*num+2], RATERES) + 1) * RATERES;
    }
  }
  else
  { /*--- decrement the cohort values by 1 if greater than zero ---*/
    sptr->deathrate[stepcnt][2*num+1] =
      (IDIV(sptr->deathrate[stepcnt][2*num+1], RATERES) - 1) * RATERES;

    if (sptr->deathrate[stepcnt][2*num+1] < 0)
    { 
      sptr->deathrate[stepcnt][2*num+1] = 0;
    }
      
    if (num != 8)
    { /* 80+ is only one cohort */
      sptr->deathrate[stepcnt][2*num+2] =
        (IDIV(sptr->deathrate[stepcnt][2*num+2], RATERES) - 1) * RATERES;
      if (sptr->deathrate[stepcnt][2*num+2] < 0)
        sptr->deathrate[stepcnt][2*num+2] = 0;
    }
  }

  /* Reconstruct the value to display. */
  sum = rebuild_deathrate(num, sptr, male, female, stepcnt);
  if (sum <= 10)
    Drag_Used = TRUE; /* checked by walk-thru tutor */
  
/*------------------------------------------------------------------------------
  MAY PUT THIS IN LATER
  /--- handle enabling, disabling of buttons ---/
  if (sptr->deathrate[stepcnt][2*num+1] <= 0)
  {
    if ((num == 8) || (sptr->deathrate[stepcnt][2*num+2] <= 0))
    {
      strcpy(tmpstr, control.control.name);
      tmpstr[strlen(tmpstr)-1] = 'm';
      GSdisablebutton(control.gname, tmpstr, DRAW); /--- disable '-' ---/
    }
  }
  else if (sptr->deathrate[stepcnt][2*num+1] >= MAXRATE)
  {
    if ((num == 8) || (sptr->deathrate[stepcnt][2*num+2] >= MAXRATE))
    {
      strcpy(tmpstr, control.control.name);
      tmpstr[strlen(tmpstr)-1] = 'p';
      GSdisablebutton(control.gname, tmpstr, DRAW); /--- disable '+' ---/
    }
  }
  else
  {
    if (old_deathrate_1 <= 0)
    {
      if ((num == 8) || (old_deathrate_2 <= 0))
      {
        strcpy(tmpstr, control.control.name);
        tmpstr[strlen(tmpstr)-1] = 'm';
        GSenablebutton(control.gname, tmpstr, DRAW); /--- disable '-' ---/
      }
    }
    else if (old_deathrate_1 >= MAXRATE)
    {
      if ((num == 8) || (old_deathrate_2 >= MAXRATE))
      {
        strcpy(tmpstr, control.control.name);
        tmpstr[strlen(tmpstr)-1] = 'p';
        GSenablebutton(control.gname, tmpstr, DRAW); /--- disable '+' ---/
      }
    }
  }
-------------------------------------------------------------------------------*/

  /* Display the new rate value */
  GSsetcurrentnamedwindow(Displayname[Cdisp]);
  GSgetfieldrect ("DEATHRATE", &x, &y, &wd, &ht, &color);
  GSwclearrect (x, y + 1 + (8-num)*(bh-1), wd, SMALL+1);
  GSsetfont (SMALL);
  GSsetcolor(color);
  GSwwriteint(3, x, y + SMALL + (8-num)*(bh-1), sum);

  /* Display the new life expectancy */
  GSsetcurrentnamedwindow (Lifexname[Cdisp]);
  GSgetfieldrect("LIFEX", &x, &y, &wd, &ht, &color);
  GSwclearrect(x, y, wd, ht);
  GSsetfont (SMALL);
  GSsetcolor(color);
  sptr->lifex_targ_val[sptr->stepcnt] =
    get_life_expect(sptr->deathrate[stepcnt]);
  GSwwritereal(4, 1, x, y-1+SMALL,
               (double)sptr->lifex_targ_val[sptr->stepcnt]/LETFRRES);
  GSclick();
  return(TRUE);
}


/*-------------------------------------------------------------------------------
  Modify the infant mortality rate 
-------------------------------------------------------------------------------*/
boolean modifyinfantrate(void)
{
  IPStatetype *sptr;
  int         stepcnt;
  int         cursx, cursy;
  int         x, y, ht, wd; /* display rectangle values */
  GSColor     color; /* display color */
  ControlType control;

  GSgetlastcontrol(&control);
  Cdisp = checkwindow(control.gname);
  if (Cdisp == DISPLAY1)
  {
    GSsetcurrentnamedwindow(control.gname);
  }
  else if (Cdisp == DISPLAY2)
  {
    GSsetcurrentnamedwindow(control.gname);
  }
  else
  {
    GSelog(0, "ERROR - in modifybirthrate with window |%s|\n", control.gname);
    return(FALSE);
  }
  GSgetcursorxy(&cursx, &cursy);
  sptr = IPState[Cdisp][Display[Cdisp].curr_sim];
  stepcnt = sptr->stepcnt;

  /* Increment or decrement as appropriate. */
  if (cursx > MODIFYBREAK)
  { /* increment the value by 1 if still less than 500 */
/*
    if (sptr->deathrate[stepcnt][0] < RATERES)
    {
      strcpy(tmpstr, control.control.name);
      tmpstr[strlen(tmpstr)-1] = 'm';
      GSenablebutton(control.gname, tmpstr, DRAW); 
    }
*/
    sptr->deathrate[stepcnt][0] =
      (IDIV(sptr->deathrate[stepcnt][0], RATERES) + 1) * RATERES;
    if (sptr->deathrate[stepcnt][0] >= MAXRATE)
    {
      sptr->deathrate[stepcnt][0] = MAXRATE;
/*      GSdisablebutton(control.gname, control.control.name, DRAW);*/
    }
/*
    if (sptr->deathrate[stepcnt][0] > MAXRATE)
    {
      sptr->deathrate[stepcnt][0] = MAXRATE;
      GSbell();
    }
*/
  }
  else
  { /* decrement the value by 1 if greater than zero */
    if (sptr->deathrate[stepcnt][0] < RATERES)
    {
      sptr->deathrate[stepcnt][0] = 0;
/*      GSbell();*/
    }
    else
      sptr->deathrate[stepcnt][0] =
        (IDIV(sptr->deathrate[stepcnt][0], RATERES) - 1) * RATERES;
/*
    if (sptr->deathrate[stepcnt][0] >= MAXRATE)
    {
      strcpy(tmpstr, control.control.name);
      tmpstr[strlen(tmpstr)-1] = 'p';
      GSenablebutton(control.gname, tmpstr, DRAW); 
    }
*/
/*
    sptr->deathrate[stepcnt][0] =
      (IDIV(sptr->deathrate[stepcnt][0], RATERES) - 1) * RATERES;
    if (sptr->deathrate[stepcnt][0] < RATERES)
    {
      sptr->deathrate[stepcnt][0] = 0;
      GSdisablebutton(control.gname, control.control.name, DRAW);
    }
*/
/*
    if (sptr->deathrate[stepcnt][0] < RATERES)
    {
      sptr->deathrate[stepcnt][0] = 0;
      GSbell();
    }
*/
/*
    else
    {
      if (sptr->deathrate[stepcnt][0] >= MAXRATE)
      {
        GSenablebutton(control.gname, control.control.name, DRAW);
      }
    }
*/
  }

  /* Display the new value */
  GSsetcurrentnamedwindow(Displayname[Cdisp]);
  GSgetfieldrect("INFRATE", &x, &y, &wd, &ht, &color);
  GSwclearrect(x, y, wd, ht);
  GSsetfont(SMALL);
  GSsetcolor(color);
  GSwwriteint(3, x, y+SMALL,
              IDIV(sptr->deathrate[stepcnt][0], RATERES));

  /* Display the new life exepectancy */
  GSsetcurrentnamedwindow(Lifexname[Cdisp]);
  GSgetfieldrect("LIFEX", &x, &y, &wd, &ht, &color);
  GSwclearrect(x, y, wd, ht);
  GSsetfont (SMALL);
  GSsetcolor(color);
  sptr->lifex_targ_val[sptr->stepcnt] =
                get_life_expect(sptr->deathrate[stepcnt]);
  GSwwritereal(4, 1, x, y-1+SMALL,
               (double)sptr->lifex_targ_val[sptr->stepcnt]/LETFRRES);
  GSclick();
  return(TRUE);
}

/*-----------------------------------------------------------------------------
  gfformat - associated with popup menu which gives user a choice of graphic
             file formats to dump to.
-----------------------------------------------------------------------------*/
boolean gfformat (
  void
)
{
  GSdisplayhelpmessage(18);
  return (TRUE);
}

/*-------------------------------------------------------------------------------
  Do nothing.  Typically the action for a popup menu call.
-------------------------------------------------------------------------------*/
boolean standard_menu(void)
{
  Savemsg = GSgetcurrmsg();
  GSdisplayhelpmessage(14);
  return(TRUE);
}

/*-------------------------------------------------------------------------------
  Do nothing.  Typically the action for a popup menu call.
-------------------------------------------------------------------------------*/
boolean no_op(void)
{
  return(TRUE);
}


/*-------------------------------------------------------------------------------
  OK button from the popup message window.
-------------------------------------------------------------------------------*/
boolean okay(void)
{
  DumpResultsActive = FALSE;
  AlertNumber = 0;
  reset_screen(FALSE);
  return(TRUE);
}


/*-------------------------------------------------------------------------------
  OK button from the TFR set window.
-------------------------------------------------------------------------------*/
boolean okayfert(void)
{
  IPStatetype *sptr = IPState[Cdisp][Display[Cdisp].curr_sim];
  IPsaveyear_t *syptr = IPsaveyear[Cdisp][Display[Cdisp].curr_sim];
  SetFertActive = FALSE;

  sptr->tfr_targ_year[sptr->stepcnt] =
    syptr->tfr_targ_year[sptr->stepcnt];
  if (sptr->year == sptr->tfr_targ_year[sptr->stepcnt])
    calc_sim_tfr_variant(sptr); /* immediate change - update rates */
  reset_screen(TRUE);
  return(TRUE);
}


/*-------------------------------------------------------------------------------
  OK button from the life expectancy set window.
-------------------------------------------------------------------------------*/
boolean okaylife(void)
{
  IPStatetype *sptr = IPState[Cdisp][Display[Cdisp].curr_sim];
  IPsaveyear_t *syptr = IPsaveyear[Cdisp][Display[Cdisp].curr_sim];
  SetLifeExActive = FALSE;

  sptr->lifex_targ_year[sptr->stepcnt] =
    syptr->lifex_targ_year[sptr->stepcnt];
  if (sptr->year == sptr->lifex_targ_year[sptr->stepcnt])
    calc_sim_life_variant(sptr);
  reset_screen(TRUE);
  return(TRUE);
}


/*-------------------------------------------------------------------------------
  OK button from the migration set window. 
-------------------------------------------------------------------------------*/
boolean okaymig(void)
{
  IPStatetype *sptr = IPState[Cdisp][Display[Cdisp].curr_sim];
  IPsaveyear_t *syptr = IPsaveyear[Cdisp][Display[Cdisp].curr_sim];
  SetMigActive = FALSE;

  sptr->mig_targ_year[sptr->stepcnt] =
    syptr->mig_targ_year[sptr->stepcnt];
  if (sptr->year == sptr->mig_targ_year[sptr->stepcnt])
    calc_sim_mig_variant(sptr); /* immediate change - update rates */
  reset_screen(TRUE);
  return(TRUE);
}


/*-------------------------------------------------------------------------------
  Options button.
-------------------------------------------------------------------------------*/
boolean options(int x, int y)
{
  ControlType control;

  x = x; /*--- to keep compiler happy ---*/
  y = y; /*--- to keep compiler happy ---*/
  GSgetlastcontrol(&control);
  if ((Cdisp = checkwindow(control.gname)) < 0) return(FALSE);
  
  if (!InWalkThru)
  {
    if (Display[Cdisp].reset_permit)
      GSenablemenuitem("options", "p5");
    else
      GSdisablemenuitem("options", "p5");
    if (Display[Cdisp].modify_permit)
    {
      if (Display[Cdisp].modify_flag)
      {
        GSenablemenuitem("options", "p4");
        GSinvisiblemenuitem("options", "p3");
      }
      else
      {
        GSenablemenuitem("options", "p3");
        GSinvisiblemenuitem("options", "p4");
      }
    }
    else
    {
      GSdisablemenuitem("options", "p3");
      GSinvisiblemenuitem("options", "p4");
    }
    if (Display[Cdisp].scenario_permit)
      GSenablemenuitem("options", "p2");
    else
      GSdisablemenuitem("options", "p2");
    if (Display[Cdisp].alt_permit)
      GSenablemenuitem("options", "p1");
    else
      GSdisablemenuitem("options", "p1");
  }
  if (!InWalkThru)
    GSdisplayhelpmessage(2);
  return(TRUE);
}


/*-------------------------------------------------------------------------------
  Initiate Options help screen
-------------------------------------------------------------------------------*/
boolean optionscmd(void)
{
  boolean return_value;

  HelpWinActive = TRUE;
  sprintf(Helpfile, "%s%s", Infopath, OPTIONS_HLP);
  return_value = GSdohelpscreens(Helpfile, FirstPage, Use_bstore, 10);
  if (MapSelect)
  {
#if 0
    drawuparrow(DISABLED, FALSE);
    drawdownarrow(DISABLED, FALSE);
#endif
  }
  return (return_value);
}


/*-------------------------------------------------------------------------------
  reset the current country window to initial state.
-------------------------------------------------------------------------------*/
boolean resetsimulation(void)
{
  GSdisablebutton (Displayname[Cdisp], "unsimulate", DRAW);
  redraw_display(Cdisp);
  GSdisplayhelpmessage(-1);
  Display[Cdisp].modify_permit = TRUE; /* allow toggling modify rates */
  init_display_data(Cdisp);
  update_display(Cdisp);
  if (!InWalkThru)
    GSdisplayhelpmessage(ParmSavemsg = 4);
  return(TRUE);
}


/*-------------------------------------------------------------------------------
 Pop up a map in which the user can select a country to appear in
   one of the display windows
-------------------------------------------------------------------------------*/
boolean selectcountrymap(void)
{

  MapSelect = TRUE;
  Pickno = -1;
  scrollselectcountry();

  if (!InWalkThru)
  {
    MapSelectSavemsg = GSgetcurrmsg();
    GSdisplayhelpmessage(3);
  }
  return (TRUE);
}


/*-------------------------------------------------------------------------------
 Initiate Set TFR box.
-------------------------------------------------------------------------------*/
boolean setfert(void)
{
  int x, y, wd, ht;
  GSColor color;
  IPStatetype *sptr;
  ControlType control;

  GSgetlastcontrol(&control);
  ParmSavemsg = GSgetcurrmsg();
  if((Cdisp = checkwindow(control.gname)) < 0) return(FALSE);
  sptr = IPState[Cdisp][Display[Cdisp].curr_sim];
  SetFertActive = TRUE;

  GSgetfieldrect("SETWINORG", &x, &y, &wd, &ht, &color);
  if (Cdisp == DISPLAY1)
    GSsetwindowspecs ("setfert", &x, &y, NULL, NULL);
  else
    GSsetwindowspecs ("setfert", &wd, &ht, NULL, NULL);
  if (sptr->tfr_targ_year[sptr->stepcnt] <= sptr->year)
    GSdisablebutton("setfert", "decfyear", NO_DRAW);
  if (sptr->tfr_targ_val[sptr->stepcnt] <= 50)
    GSdisablebutton("setfert", "decfert", NO_DRAW);
  else
    GSenablebutton("setfert", "decfert", NO_DRAW);
  if (sptr->tfr_targ_val[sptr->stepcnt] >= 990)
    GSdisablebutton("setfert", "incfert", NO_DRAW);
  else
    GSenablebutton("setfert", "incfert", NO_DRAW);

  GSdrawnamedwindow("setfert", Use_bstore, POPUP);
  display_fert_win(NO_DRAW);

  if (!InWalkThru)
    GSdisplayhelpmessage(10);
  return(TRUE);
}


/*-------------------------------------------------------------------------------
  Initiate Set Life Expectancy box. 
-------------------------------------------------------------------------------*/
boolean setlifeex(void)
{
  int x, y, wd, ht;
  GSColor color;
  IPStatetype *sptr;
  ControlType control;

  GSgetlastcontrol(&control);
  ParmSavemsg = GSgetcurrmsg();
  if((Cdisp = checkwindow(control.gname)) < 0) return(FALSE);
  sptr = IPState[Cdisp][Display[Cdisp].curr_sim];
  SetLifeExActive = TRUE;

  GSgetfieldrect("SETWINORG", &x, &y, &wd, &ht, &color);
  if (Cdisp == DISPLAY1)
    GSsetwindowspecs ("setlife", &x, &y, NULL, NULL);
  else
    GSsetwindowspecs ("setlife", &wd, &ht, NULL, NULL);
  if (sptr->lifex_targ_year[sptr->stepcnt] <= sptr->year)
    GSdisablebutton("setlife", "declyear", NO_DRAW);
  if (sptr->lifex_targ_val[sptr->stepcnt] >= 9990)
    GSdisablebutton("setlife", "inclifeexp", NO_DRAW);
  else
    GSenablebutton("setlife", "inclifeexp", NO_DRAW);
  if (sptr->lifex_targ_val[sptr->stepcnt] <= 2000)
    GSdisablebutton("setlife", "declifeexp", NO_DRAW);
  else
    GSenablebutton("setlife", "declifeexp", NO_DRAW);

  GSdrawnamedwindow("setlife", Use_bstore, POPUP);
  display_lifeex_win(NO_DRAW);

  if (!InWalkThru)
    GSdisplayhelpmessage(10);
  return(TRUE);
}

/*-------------------------------------------------------------------------------
  Decrement map scale 
-------------------------------------------------------------------------------*/
boolean decmapscale(void)
{
  return(changemapscale(-1));
}


/* Increment map scale  */
boolean incmapscale(void)
{
  return(changemapscale(1));
}



/* Init set migration box */
boolean setmig(void)
{
  int x, y, wd, ht;
  GSColor color;
  IPStatetype *sptr;
  ControlType control;

  GSgetlastcontrol(&control);
  ParmSavemsg = GSgetcurrmsg();
  if((Cdisp = checkwindow(control.gname)) < 0) return(FALSE);
  sptr = IPState[Cdisp][Display[Cdisp].curr_sim];
  SetMigActive = TRUE;

  GSgetfieldrect("SETWINORG", &x, &y, &wd, &ht, &color);
  if (Cdisp == DISPLAY1)
    GSsetwindowspecs ("setmig", &x, &y, NULL, NULL);
  else
    GSsetwindowspecs ("setmig", &wd, &ht, NULL, NULL);
  if (sptr->mig_targ_year[sptr->stepcnt] <= sptr->year)
      GSdisablebutton("setmig", "decmyear", NO_DRAW);
  if (sptr->mig_targ_val[sptr->stepcnt] <= -100000L)
    GSdisablebutton("setmig", "decmig", NO_DRAW);
  else
    GSenablebutton("setmig", "decmig", NO_DRAW);
  if (sptr->mig_targ_val[sptr->stepcnt] >= 100000L)
    GSdisablebutton("setmig", "incmig", NO_DRAW);
  else
    GSenablebutton("setmig", "incmig", NO_DRAW);

  GSdrawnamedwindow("setmig", Use_bstore, POPUP);

  display_mig_win(NO_DRAW);

  if (!InWalkThru)
    GSdisplayhelpmessage(10);

  return(TRUE);
}


/* Process "Simulate" button. */
boolean simulate(void)
{
  IPStatetype *sptr;
  ControlType control;

  GSgetlastcontrol(&control);
  if((Cdisp = checkwindow(control.gname)) < 0) return(FALSE);
  sptr = IPState[Cdisp][Display[Cdisp].curr_sim];
  sptr->stepcnt++;
  dosim(sptr);
  if (sptr->stepcnt == 1)
    if (!InWalkThru)
      GSenablebutton (Displayname[Cdisp], "unsimulate", DRAW);
  if (sptr->stepcnt == NUM_POINTS-1)
  {  /* Don't let user change things when at end of simulation */
    Display[Cdisp].modify_permit = FALSE;
    if (!InWalkThru)
      GSdisplayhelpmessage(ParmSavemsg = 9);
    if (Display[Cdisp].modify_flag)
    {
      GSsetdeactivenamedwindow(Birthwinname[Cdisp]);
      GSsetdeactivenamedwindow(Deathwinname[Cdisp]);
      GSsetdeactivenamedwindow(Infantwinname[Cdisp]);
    }
    else
    {
      GSdisablebutton(TFRname[Cdisp], "setfert", DRAW);
      GSdisablebutton(Lifexname[Cdisp], "setlife", DRAW);
    }
    GSdisablebutton(Migname[Cdisp], "setmig", DRAW);
    GSdisablebutton(Displayname[Cdisp], "pyrgraph", NO_DRAW);
	GSdisablebutton(Displayname[Cdisp], "simulate", DRAW);
	Drag_Used = TRUE;		/* walkthru checks this flag */
  } /* if (sptr->stepcnt == NUM_POINTS-1) */
  setup_bargraph();
  update_display(Cdisp);
  writesimstats (sptr);
  return(TRUE);
}


/* Initiate "Technical Help" screen */
boolean techcmd(void)
{
  boolean return_value;

  HelpWinActive = TRUE;
  sprintf(Helpfile, "%s%s", Infopath, TECH_HLP);
  return_value = GSdohelpscreens(Helpfile, FirstPage, Use_bstore, 10);
  if (MapSelect)
  {
#if 0
    drawuparrow(DISABLED, FALSE);
    drawdownarrow(DISABLED, FALSE);
#endif
  }
  return (return_value);
}


/* Initiate "Version" help screen */
boolean versioncmd(void)
{
  boolean return_value;

  HelpWinActive = TRUE;
  sprintf(Helpfile, "%s%s", Infopath, POP_HLP);
  return_value = GSdohelpscreens(Helpfile, FirstPage, Use_bstore, 10);
  if (MapSelect)
  {
#if 0
    drawuparrow(DISABLED, FALSE);
    drawdownarrow(DISABLED, FALSE);
#endif
  }
  return (return_value);
}

/* Process "Unsimulate" button. */
boolean unsimulate(void)
{
  IPStatetype *sptr;
  ControlType control;

  GSgetlastcontrol(&control);
  if((Cdisp = checkwindow(control.gname)) < 0) return(FALSE);
  sptr = IPState[Cdisp][Display[Cdisp].curr_sim];
  if (sptr->stepcnt == NUM_POINTS-1)
  {
    if (!InWalkThru)
      GSdisplayhelpmessage(ParmSavemsg = 4);
    Display[Cdisp].modify_permit = TRUE;
    if (Display[Cdisp].modify_flag)
    {
      GSsetactivenamedwindow(Birthwinname[Cdisp]);
      GSsetactivenamedwindow(Deathwinname[Cdisp]);
      GSsetactivenamedwindow(Infantwinname[Cdisp]);
      GSenablebutton (Migname[Cdisp], "setmig", DRAW);
      GSenablebutton (Displayname[Cdisp], "pyrgraph", NO_DRAW);
      GSenablebutton (Displayname[Cdisp], "simulate", DRAW);
    }
    else
    {
      GSenablebutton(TFRname[Cdisp], "setfert", DRAW);
      GSenablebutton(Lifexname[Cdisp], "setlife", DRAW);
      GSenablebutton(Migname[Cdisp], "setmig", DRAW);
      GSenablebutton(Displayname[Cdisp], "pyrgraph", NO_DRAW);
      GSenablebutton(Displayname[Cdisp], "simulate", DRAW);
    }
  } /* if (sptr->stepcnt == NUM_POINTS-1) */
  if (sptr->stepcnt > 0)
    {
    sptr->stepcnt--;
    sptr->year -= 5;
    if (sptr->stepcnt == 0)
      {
        if (!InWalkThru)
          GSdisplayhelpmessage(4);
        GSdisablebutton (Displayname[Cdisp], "unsimulate", DRAW);
      }
    setup_bargraph();
    update_display(Cdisp);
    writesimstats (sptr);
	}
  else
    {
    GSelog(0, "ERROR - tried to unsimulate when at 1990.");
    }
  return(TRUE);
}


/***************************************************************************
  Save the current simulation state to the SaveFile
***************************************************************************/
boolean save_simulation (void)
{
  return(savenamedfile(SaveFile));
}


/***************************************************************************
  Save the current simulation state to the TUTOR_SAVE file
***************************************************************************/
boolean save_tutor (void)
{
  savenamedfile(TUTOR_SAVE);
  GSquit(NULL);
  return (TRUE);
}


/***************************************************************************
  Load the simulation state from SaveFile.
  If SaveFile is the tutorfile, delete it.
***************************************************************************/
boolean load_simulation (void)
{
  GSdeactivatefunction ("load_simulation");

  if (!loadnamedfile(SaveFile))
  {
    GSelog(0, "Simulation from |%s| not loaded. \n", SaveFile);
    GSalert(10, Use_bstore, "loadalert1", "loadalert2",
                "loadalert3", "loadalert4", "");
    AlertNumber = 2;
  }

  /* delete the tutorial save file, if exists */
  if (Tutorfile)
    if ( remove(TUTOR_SAVE) != 0 )
	  GSelog(0, "Error, could not delete |%s|", TUTOR_SAVE);

  return(TRUE);
}

/***********************************************************************
* Confirm quit.  Display quit window. *
***********************************************************************/
boolean confirmquit(void)
{
  GSdisplayhelpmessage(15);
  GSdrawnamedwindow("quit", Use_bstore, POPUP);
  QuitWinActive = TRUE;
  if (MapSelect)
  {
#if 0
    drawuparrow(DISABLED, FALSE);
    drawdownarrow(DISABLED, FALSE);
#endif
  }
  return TRUE;
}

/***********************************************************************
* Cancel quit.  Erase quit window and return to program. *
***********************************************************************/
boolean cancelquit(void)
{
  GSremovenamedwindow("quit");
  QuitWinActive = FALSE;
  if (!Use_bstore) redraw_all();
  if (MapSelect)
  {
  /*  drawuparrow(UpArrowStatus, FALSE);
    drawdownarrow(DnArrowStatus, FALSE);*/
  }
  GSdisplayhelpmessage (-2);
  return TRUE;
}

/***********************************************************************
  Print screen to file
***********************************************************************/
boolean dumpscreen(void)
{
  BigString msgstring;

  GSremovenamedwindow("dump");
  GSdisplayhelpmessage(Savemsg);
  DumpWinActive = FALSE;
  if (!Use_bstore)
    redraw_all();
  strcpy (DumpFileName, "intpop");
  if (GraphFileFmt != NO_FILE)
    DumpSuccess = GSdumpscreen(DumpFileName, GraphFileFmt);
  else return FALSE;

  GSdisplayhelpmessage(10);
  DumpResultsActive = TRUE;
  GSdrawnamedwindow("alert", Use_bstore, POPUP);
  GSsetcolor(GSconvertcolor("black"));
  GSsetfont(LARGE);

  if (DumpSuccess)
  {
    sprintf(msgstring, "The screen was saved to file:    %s", DumpFileName);
    GSwwritemsg(6, 30, msgstring);
    return TRUE;
  }
  else
  {
    sprintf(msgstring, "An error occurred saving the screen.");
    GSwwritemsg(6, 15, msgstring);
    sprintf(msgstring, "No file was saved.");
    GSwwritemsg(6, 30, msgstring);
    sprintf(msgstring, "Check disk space and file permissions.");
    GSwwritemsg(6, 45, msgstring);
    GSbell();
    return FALSE;
  }
}

boolean canceldump (
  void
)
{
  GSremovenamedwindow("dump");
  DumpWinActive = FALSE;
  WarnedNotToCoverWin = FALSE;
  if (!Use_bstore) redraw_all();
  GSdisplayhelpmessage (Savemsg);
  return TRUE;
}

void highlightfield (
  BigString  window,
  BigString  field,
  GSColor    bgcolor,
  GSColor    topleft,
  GSColor    bottomright
)
{
  int fx, fy, fwd, fht;

  GSgetfieldrect(field, &fx, &fy, &fwd, &fht, NULL);
  GSsetcurrentnamedwindow(window);
  GSsetcolor(bgcolor);
  GSwfillrect(fx, fy, fwd, fht);
  GSsetcolor(topleft);
  GSwdrawline(fx-1, fy-1, fx+fwd, fy-1);
  GSwdrawline(fx-2, fy-2, fx+fwd+1, fy-2);
  GSwdrawline(fx-1, fy-1, fx-1, fy+fht);
  GSwdrawline(fx-2, fy-2, fx-2, fy+fht+1);
  GSsetcolor(bottomright);
  GSwdrawline(fx, fy+fht, fx+fwd, fy+fht);
  GSwdrawline(fx-1, fy+fht+1, fx+fwd+1, fy+fht+1);
  GSwdrawline(fx+fwd, fy, fx+fwd, fy+fht);
  GSwdrawline(fx+fwd+1, fy-1, fx+fwd+1, fy+fht+1);
}

/*-------------------------------------------------------------------------------
  set_format: set a graphics file format for screen dump in response to user
              selection from GFformat popup menu
-------------------------------------------------------------------------------*/
boolean set_format (
  int item_n
)
{
  int x, y;
  GSColor color;

  GSgetfieldrect("filetype", &x, &y, NULL, NULL, &color);
  highlightfield("dump", "filetype", GSconvertcolor("lgray"),
                 GSconvertcolor("black"), GSconvertcolor("white"));
  GSsetcolor(color);
  GSsetfont(LARGE);
  switch (item_n) {
    case 0:
      GraphFileFmt = BMP_16;
      GSwwritenamedmsg(x+5, y+SMALL+2, "bmp16msg");
      break;
    case 1:
      GraphFileFmt = PS_COLOR;
      GSwwritenamedmsg(x+5, y+SMALL+2, "pscolormsg");
      break;
    case 2:
      GraphFileFmt = PS_GRAY;
      GSwwritenamedmsg(x+5, y+SMALL+2, "psgraymsg");
      break;
    case 3:
      GraphFileFmt = GIF_16;
      GSwwritenamedmsg(x+5, y+SMALL+2, "gif16msg");
      break;
  }
  return (TRUE);
}

/*-------------------------------------------------------------------------------
  Displays the "dump" window, warning the user that this will take a long
  time. The user is asked to confirm his or her wish to dump the screen to file
-------------------------------------------------------------------------------*/
boolean confirmdump (
  void
)
{
  int x, y;
  GSColor color;

  GSdisplayhelpmessage(17);
  DumpWinActive = TRUE;
  GSdrawnamedwindow("dump", Use_bstore, POPUP);
  GSgetfieldrect("filetype", &x, &y, NULL, NULL, &color);
  highlightfield("dump", "filetype", GSconvertcolor("lgray"),
                 GSconvertcolor("black"), GSconvertcolor("white"));
  GSsetcolor(color);
  GSsetfont(LARGE);
  switch (GraphFileFmt) {
    case BMP_16:
      GSwwritenamedmsg(x+5, y+SMALL+2, "bmp16msg");
      break;
    case GIF_16:
      GSwwritenamedmsg(x+5, y+SMALL+2, "gif16msg");
      break;
    case PS_GRAY:
      GSwwritenamedmsg(x+5, y+SMALL+2, "psgraymsg");
      break;
    case PS_COLOR:
      GSwwritenamedmsg(x+5, y+SMALL+2, "pscolormsg");
      break;
    case NO_FILE: default:
      GSwwritenamedmsg(x+5, y+SMALL+2, "nofilemsg");
      break;
  }
  if (GraphFileFmt != NO_FILE)
  {
/*    GSgetfieldrect("dumpwarn1", &x, &y, NULL, NULL, &color);
    GSsetcolor(color);
    GSwwritenamedmsg(x, y, "dumpwarn1");
    GSgetfieldrect("dumpwarn2", &x, &y, NULL, NULL, &color);
    GSsetcolor(color);
    GSwwritenamedmsg(x, y, "dumpwarn2");
    GSgetfieldrect("dumpwarn3", &x, &y, NULL, NULL, &color);
    GSsetcolor(color);
    GSwwritenamedmsg(x, y, "dumpwarn3");*/
  }
  return TRUE;
}

/* Arrays associating string names with function pointers for buttons */
IntrFunction FuncNames[] = {
  {"alternatesim",  alternatesim},
  {"birthdeathrate",  birthdeathrate},
  {"canceldump", canceldump},
  {"cancelquit", cancelquit},
  {"controlscmd",  controlscmd},
  {"confirmdump", confirmdump},
  {"confirmquit", confirmquit},
  {"dumpscreen", dumpscreen},
  {"decfert",  decfert},
  {"decfyear",  decfyear},
  {"declifeexp",  declifeexp},
  {"declyear",  declyear},
  {"decmapscale",  decmapscale},
  {"decmig",  decmig},
  {"decmyear",  decmyear},
  {"do_intro",  do_intro},
  {"done",  done},
  {"doquit",  doquit},
  {"dragbars",  dragbars},
  {"draglargemap",  draglargemap},
  {"dragsmallmap",  dragsmallmap},
  {"expectedsim",  expectedsim},
  {"file",  standard_menu},
  {"geocmd",  geocmd},
  {"glossary", standard_menu},
  {"gfformat", gfformat},
  {"handlesearch", (boolean(*)())handlesearch},
  {"help",  standard_menu},
  {"helpnext",  helpnext},
  {"helpprev",  helpprev},
  {"incfert",  incfert},
  {"incfyear",  incfyear},
  {"inclifeexp",  inclifeexp},
  {"inclyear",  inclyear},
  {"incmapscale",  incmapscale},
  {"incmig",  incmig},
  {"incmyear",  incmyear},
  {"info",  standard_menu},
  {"instrdone",  instrdone},
  {"introhelpcmd",  introhelpcmd},
  {"keyfunc", (boolean(*)())handlesearch},
  {"load_simulation",  load_simulation},
  {"maincmd",  maincmd},
  {"mapcancel",  mapcancel},
  {"mapdone",  mapdone},
  {"modifybirthrate",  modifybirthrate},
  {"modifydeathrate",  modifydeathrate},
  {"modifyinfantrate",  modifyinfantrate},
  {"okay",  okay},
  {"okayfert",  okayfert},
  {"okaylife",  okaylife},
  {"okaymig",  okaymig},
  {"options",  options},
  {"optionscmd",  optionscmd},
  {"qtwalk", quitwalkthru},
  {"quit", standard_menu},
  {"redraw_all", (boolean(*)())redraw_all},
  {"resetsimulation",  resetsimulation},
  {"save_simulation",  save_simulation},
  {"save_tutor", save_tutor},
  {"scrolldraggetcountry",  scrolldraggetcountry},
  {"scrollselectcountry",  scrollselectcountry},
  {"scrolldnline", scrolldnline},
  {"scrollupline", scrollupline},
  {"scrollbar", scrollbar},
  {"selectcountrymap",  selectcountrymap},
  {"setfert",  setfert},
  {"set_format",  set_format},
  {"setlifeex",  setlifeex},
  {"setmig",  setmig},
  {"simulate",  simulate},
  {"techcmd",  techcmd},
  {"unsimulate",  unsimulate},
  {"versioncmd",  versioncmd},
  {"walkthru", walkthru},
  {"walkthruinit", walkthruinit},
  {"walkthrucont", no_op},
};

int NumFuncs = GSnumifuncs(FuncNames);
