/***********************************************************************
International Population module - Walkthrough Tutorial module
***********************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "ipincl.h"

/* Status values for walkthrough function.  GS event loop always passes
 0 as the third parameter of an activated funtion */
#define WT_LOOP 0
#define WT_QUIT 10
#define READTIME 1
#define DEMOSEARCH "eu"
#define WALKTHRU_SAVE "ip_walk.sav"
#define WALKTHRU_INIT "ip_wt.ini"


#define NEXT(x)   (x++)		/* increment x */
#define PREV(x)   (x--)     /* decrement x */

/*  Enumeration of steps in tutorial.  These should be listed in the order
	in which they are to be executed.  */
typedef enum stagetypes
{
    WT_NONE,
	INTRO,
	PRESSINTRODONE,
	PRESSOPTIONS,
	PRESSSELECT,
    CONTINUE1,
/*	MAPDESCRIBE, */
	NOTICESELECT,
/*    CONTINUE2, */
	CONTINUE3,
	DRAGTOPRECT,
	ZOOMIN,
	ZOOMOUT,
	PICKFROMLIST,
	PRESSDNARROW,
    PRESSUPARROW,
	CONTINUE4,
	MOVESCROLLBUTT,
	TYPENAME,
	ENDMAPSELECT,
	PRESSMAPDONE,
	CONTINUE5,
	CONTINUE6,
	PRESSSIM,
	PRESSUNSIMAGAIN,
	GOTO2050,
	PRESSOPTIONSAGAIN,
	PRESSRUNANOTHER,
	CONTINUE7,
	PRESSTOTFERT,
	PRESSDECFERT,
	INCFYEAR2010,
	PRESSFERTDONE,
	GOTO2050AGAIN,
	CONTINUE8,
	PRESSOPTIONS3,
	PRESSRESETSIM,
	PRESSOPTIONS4,
	PRESSVIEWBUTTONS,
	CONTINUE9,
	PRESSDEATHMINUS,
	CONTINUE10,
	DRAGCOHORT,
	GOTO2050FINAL,
	CONTINUE11,

	QUITTUT
} StageType;


static StageType Stage = WT_NONE;  /* Global, identifies current stage of tutorial */
static int WTSaveMessage;

/***********************************************************************
  initialization function for walkthrough tutorial
***********************************************************************/
boolean walkthruinit(void)
{
	BigString filename;

  GSelog(0, "Start of Walk-Through Tutorial\n");

  if (MapSelect)
  {
    MapSelect = FALSE;
    mapcancel();
  }
  /*  Save current state of simulations */
  sprintf(filename, "%s%s", Home, WALKTHRU_SAVE);
  savenamedfile(filename);
  WTSaveMessage = GSgetcurrmsg();
  /*  Initialize the windows to blank */
  sprintf(filename, "%s%s", Home, WALKTHRU_INIT);
  loadnamedfile(filename);

  GSmouseoff();
  /* disable buttons that we don't want users to get at */
  if (TutorEntry)
  {
    GSinvisiblebutton("root", "file", DRAW);
    GSinvisiblebutton("root", "glossary", DRAW);
  }
  else
  {
    GSinvisiblebutton("root", "file", DRAW);
  }
  GSdisablebutton("bottom", "options", DRAW);
  /* enable Walkthru Quit button */
  GSenablebutton("root", "qtwalk", DRAW);
  GSdisablebutton("message", "wtcont", NO_DRAW);
  GSdisplayhelpmessage(90);
  GSmouseon();

  /* activate/register the walkthru function */
  /* The event loop does not turn off the mouse before it calls walkthru()
	 (to eliminate flicker and for efficiency), so all drawing routines
	 MUST be bracketted by GSmouseoff() and GSmouseon().
  */
  GSactivatefunction("walkthru");
  GSdisablemenuitem("help", "p4");		/* Do not let users reselect walkthru */

  Stage = INTRO;
  Drag_Used = FALSE;
  InWalkThru = TRUE;
  return(TRUE);
}

/***********************************************************************
  walkthrough tutorial function

Globals: Drag_Used  - set by dragrect functs.
***********************************************************************/
boolean walkthru(int x,
                 int y,
                 DragStatusType status)
{
  int i;
  BigString tempstrng;
  ControlType CurrControl;

  x = y;               /* to make the compiler shut up */
  y = x;

  if (status == WT_QUIT)
  {
    Stage = QUITTUT;
    quitwalkthru();
  }

  GSgetlastcontrol(&CurrControl);
/*	if (CurrControl.type == NONE)
		return(TRUE);
*/
/***************
 *  In the cases of this switch, we are checking to see if an instruction has
 * been done yet.  e.g. Stage == PRESSOPTIONS means that we have given the
 * instruction to 'Press the Options button' and are now waiting for that to
 * occur.  So at case PRESSOPTIONS, we check to see if the last control
 * pressed was the options button; if so, give next instruction and increment
 * the Stage, otherwise continue till it happens.
 ***************/

  switch(Stage)
  {
    case INTRO: /* actually starting with Stage == 10 */
      GSmouseoff();
      GSalert(96, Use_bstore, "introalert1", "introalert2", "introalert3",
              "introalert4", "introalert5", "introalert6",  "");
      AlertNumber = 3;
      GSmouseon();
      NEXT(Stage);
      break;

    case PRESSINTRODONE :
      if ((CurrControl.type == WINDOW) &&
          (GSstricmp(CurrControl.gname, "alert") == 0) &&
          (GSstricmp(CurrControl.control.name, "ok") == 0))
      {
        GSmouseoff();
        GSdisplayhelpmessage(91);   /* Press Options */
        GSmouseon();
        NEXT(Stage);
      }
      break;

    case PRESSOPTIONS :
      if ((CurrControl.type == WINDOW) &&
          (GSstricmp(CurrControl.gname, "top") == 0) &&
          (GSstricmp(CurrControl.control.name, "Options") == 0))
      {
        GSmouseoff();
        GSdisplayhelpmessage(92);   /* Press select region/country */
        GSmouseon();
        NEXT(Stage);
      }
      break;

    case PRESSSELECT : /* Press 'Select region/country for simulation.' */
      if ((CurrControl.type == MENU) &&
          (GSstricmp(CurrControl.gname, "options") == 0) &&
          (CurrControl.control.number == 0))
      {
        GSmouseoff();
        GSdisablebutton("map", "done", DRAW);
        GSdisablebutton("map", "cancel", DRAW);
        GSdisablebutton("map", "incmapscale", DRAW);
        GSdisablebutton("map", "decmapscale", DRAW);
        GSdisablebutton("map", "smallmap", NO_DRAW);
        GSdisablebutton("map", "largemap", NO_DRAW);
        GSdisablebutton("map", "scrollcountry", NO_DRAW);
        GSdisablebutton("map", "scrollbar", NO_DRAW);
        GSdisablebutton("map", "DnArrow", NO_DRAW);
        /*drawdownarrow(DISABLED, TRUE);*/
        GSenablebutton("message", "wtcont", NO_DRAW);
        GSclearcharfunc();
        GSdisplayhelpmessage(97);  /* there are two maps */
        GSmouseon();
        NEXT(Stage);
      }
      break;
    case CONTINUE1 : /* Press Continue for next message */
      if ((CurrControl.type == WINDOW) &&
          (GSstricmp(CurrControl.gname, "message") == 0 ) &&
          (GSstricmp(CurrControl.control.name, "wtcont") == 0))
      {
        GSdisablebutton("message", "wtcont", NO_DRAW);
        GSmouseoff();
        GSdisplayhelpmessage(93); /* now select from a map */
        GSenablebutton("map", "smallmap", NO_DRAW);
        GSenablebutton("map", "largemap", NO_DRAW);
        GSmouseon();
        GSclearlastcontrol();
        NEXT(Stage);
      }
      break;
    case NOTICESELECT :
      if (!Drag_Used)
      {
        if ((CurrControl.type == WINDOW) &&
            (GSstricmp(CurrControl.gname, "map") == 0) &&
            ((GSstricmp(CurrControl.control.name, "largemap") == 0) ||
             (GSstricmp(CurrControl.control.name, "smallmap") == 0)))
        {
          GSenablebutton("message", "wtcont", NO_DRAW);
          GSmouseoff();
          GSdisablebutton("map", "smallmap", NO_DRAW);
          GSdisablebutton("map", "largemap", NO_DRAW);
          GSdisplayhelpmessage(98);  /* see your selection */
          GSmouseon();
          NEXT(Stage);
        }
      }
      else
      {
        Drag_Used = FALSE;
        GSclearlastcontrol();
      }
      break;
    case CONTINUE3 :
      if ((CurrControl.type == WINDOW) &&
          (GSstricmp(CurrControl.gname, "message") == 0 ) &&
          (GSstricmp(CurrControl.control.name, "wtcont") == 0))
      {
        GSdisablebutton("message", "wtcont", NO_DRAW);
        GSmouseoff();
        GSdisplayhelpmessage(89); /* Drag rect in top map */
        GSenablebutton("map", "smallmap", NO_DRAW);
        GSenablebutton("map", "largemap", NO_DRAW);
        GSmouseon();
        GSclearlastcontrol();
        NEXT(Stage);
      }
      break;
    case DRAGTOPRECT :
      if (Drag_Used)
      {
        Drag_Used = FALSE;
        GSmouseoff();
        GSdisplayhelpmessage(86);  /* Use + to zoom in */
        GSenablebutton("map", "incmapscale", DRAW);
        GSmouseon();
        NEXT(Stage);
      }
      break;
    case ZOOMIN :
      if ((CurrControl.type == WINDOW) &&
          (GSstricmp(CurrControl.gname, "map") == 0) &&
          (GSstricmp(CurrControl.control.name, "incmapscale") == 0))
      {
        GSmouseoff();
        GSdisablebutton("map", "incmapscale", DRAW);
        GSenablebutton("map", "decmapscale", DRAW);
        GSdisplayhelpmessage(87);  /* Use - to zoom out */
        GSmouseon();
        NEXT(Stage);
      }
      break;
    case ZOOMOUT :
      if ((CurrControl.type == WINDOW) &&
          (GSstricmp(CurrControl.gname, "map") == 0) &&
          (GSstricmp(CurrControl.control.name, "decmapscale") == 0))
      {
        GSmouseoff();
        GSdisablebutton("map", "decmapscale", DRAW);
        GSenablebutton("map", "scrollcountry", NO_DRAW);
        GSdisplayhelpmessage(99);
        GSmouseon();
        NEXT(Stage);
      }
      break;
    case PICKFROMLIST :
      if ((CurrControl.type == WINDOW) &&
          (GSstricmp(CurrControl.gname, "map") == 0) &&
          (GSstricmp(CurrControl.control.name, "scrollcountry") == 0))
      {
        GSmouseoff();
        GSenablebutton("map", "DnArrow", NO_DRAW);
        /*drawdownarrow(ENABLED, TRUE);*/
        GSdisablebutton("map", "scrollcountry", NO_DRAW);
        GSdisplayhelpmessage(80); /* Press down arrow button */
        GSmouseon();
        NEXT(Stage);
      }
      break;
    case PRESSDNARROW :
      if ((CurrControl.type == WINDOW) &&
          (GSstricmp(CurrControl.gname, "map") == 0) &&
          (GSstricmp(CurrControl.control.name, "dnarrow") == 0))
      {
        GSmouseoff();
        GSdisablebutton("map", "DnArrow", NO_DRAW);
        /*drawdownarrow(DISABLED, TRUE);*/
        GSdisplayhelpmessage(81); /* Press down arrow again */
        GSmouseon();
        NEXT(Stage);
        /* Clear the last control struct so we can make the user
           press it again. */
        GSclearlastcontrol();
      }
      break;
    case PRESSUPARROW :
      if ((CurrControl.type == WINDOW) &&
          (GSstricmp(CurrControl.gname, "map") == 0) &&
          (GSstricmp(CurrControl.control.name, "uparrow") == 0))
      {
        GSmouseoff();
        GSenablebutton("message", "wtcont", NO_DRAW);
        GSdisablebutton("map", "UpArrow", NO_DRAW);
       /* drawuparrow(DISABLED, TRUE);*/
        GSdisablebutton("map", "DnArrow", NO_DRAW);
       /* drawdownarrow(DISABLED, TRUE);*/
        GSdisplayhelpmessage(82); /* Notice the Scroll button?*/
        GSmouseon();
        Drag_Used = FALSE;
        NEXT(Stage);
      }
      break;
    case CONTINUE4 :
      if ((CurrControl.type == WINDOW) &&
          (GSstricmp(CurrControl.gname, "message") == 0 ) &&
          (GSstricmp(CurrControl.control.name, "wtcont") == 0))
      {
        GSdisablebutton("message", "wtcont", NO_DRAW);
        GSmouseoff();
        GSdisplayhelpmessage(83);   /* Move the Scroll button. */
        GSenablebutton("map", "scrollbar", NO_DRAW);
        GSmouseon();
        GSclearlastcontrol();
        NEXT(Stage);
      }
      break;
    case MOVESCROLLBUTT :
      if (Drag_Used)
      {
        Drag_Used = FALSE;
        GSmouseoff();
        GSdisablebutton("map", "UpArrow", NO_DRAW);
       /*drawuparrow(DISABLED, TRUE);*/
        GSdisablebutton("map", "DnArrow", NO_DRAW);
       /* drawdownarrow(DISABLED, TRUE);*/
        GSdisplayhelpmessage(84); /* Type in DEMOSEARCH text */
        GScharfunc("keyfunc");
        GSmouseon();
        NEXT(Stage);
      }
      break;
    case TYPENAME :
      if (GSstricmp(searchstring, DEMOSEARCH) == 0)
      {
        GSmouseoff();
        GSdisplayhelpmessage(85);   /* Press on DEMOSEARCH */
        GSenablebutton("map", "scrollcountry", NO_DRAW);
        GSmouseon();
        Drag_Used = FALSE;
        NEXT(Stage);
      }
      break;
    case ENDMAPSELECT :
      if (Drag_Used)
      {
        GSmouseoff();
        Drag_Used = FALSE;
        GSenablebutton("map", "done", DRAW);
        GSdisplayhelpmessage(94); /* press Done to finish */
        GSmouseon();
        NEXT(Stage);
      }
      break;
    case PRESSMAPDONE :
      if ((CurrControl.type == WINDOW) &&
          (GSstricmp(CurrControl.gname, "map") == 0) &&
          (GSstricmp(CurrControl.control.name, "done") == 0))
      {
        GSmouseoff();
        GSdisablebutton("top", "Pyrgraph", NO_DRAW);
        GSdisablebutton("top", "simulate", DRAW);
        GSdisablebutton("top", "options", DRAW);
        GSdisablebutton("toptotalfert", "setfert", DRAW);
        GSdisablebutton("toplifeexp", "setlife", DRAW);
        GSdisablebutton("topmig", "setmig", DRAW);
        GSenablebutton("message", "wtcont", NO_DRAW);
        GSdisplayhelpmessage(70); /* Notice the pyramid */
        GSmouseon();
        NEXT(Stage);
      }
      break;
    case CONTINUE5 :
      if ((CurrControl.type == WINDOW) &&
          (GSstricmp(CurrControl.gname, "message") == 0 ) &&
          (GSstricmp(CurrControl.control.name, "wtcont") == 0))
      {
        GSmouseoff();
        GSdisplayhelpmessage(71); /* Notice Infant Mort */
        GSmouseon();
        GSclearlastcontrol();
        NEXT(Stage);
      }
      break;
    case CONTINUE6 :
      if ((CurrControl.type == WINDOW) &&
          (GSstricmp(CurrControl.gname, "message") == 0 ) &&
          (GSstricmp(CurrControl.control.name, "wtcont") == 0))
      {
        GSdisablebutton("message", "wtcont", NO_DRAW);
        GSmouseoff();
        GSdisplayhelpmessage(72); /* Press SIM--> */
        GSenablebutton("top", "simulate", DRAW);
        GSmouseon();
        GSclearlastcontrol();
        NEXT(Stage);
      }
      break;
    case PRESSSIM :
      if ( ((CurrControl.type == WINDOW) &&
          (GSstricmp(CurrControl.gname, "top") == 0) &&
          (GSstricmp(CurrControl.control.name, "simulate") == 0)) ||
          (IPState[Cdisp][Display[Cdisp].curr_sim]->stepcnt == NUM_POINTS-1))
      {
        GSmouseoff();
        GSenablebutton("top", "unsimulate", DRAW);
        GSdisablebutton("top", "simulate", DRAW);
        GSdisplayhelpmessage(73); /* Press <--SIM, Notice the changes */
        GSmouseon();
        GSclearlastcontrol();
        NEXT(Stage);
      }
      break;
    case PRESSUNSIMAGAIN :
      if ( ((CurrControl.type == WINDOW) &&
          (GSstricmp(CurrControl.gname, "top") == 0) &&
          (GSstricmp(CurrControl.control.name, "unsimulate") == 0)) ||
          (IPState[Cdisp][Display[Cdisp].curr_sim]->stepcnt == 0))
      {
        GSmouseoff();
        GSenablebutton("top", "simulate", DRAW);
        GSdisablebutton("top", "unsimulate", DRAW);
        GSdisplayhelpmessage(76); /* Press SIM--> to 2050 */
        GSmouseon();
        Drag_Used = FALSE;
        NEXT(Stage);
      }
      break;
    case GOTO2050 :
      if (Drag_Used)
      {
        Drag_Used = FALSE;
        GSmouseoff();
        GSdisplayhelpmessage(77); /* Press Options */
        GSenablebutton("top", "options", DRAW);
        GSdisablemenuitem("options", "p0");
        GSenablemenuitem("options", "p1");
        GSdisablemenuitem("options", "p2");
        GSdisablemenuitem("options", "p3");
        GSdisablemenuitem("options", "p4");
        GSdisablemenuitem("options", "p5");
        GSmouseon();
        NEXT(Stage);
      }
      break;
    case PRESSOPTIONSAGAIN :
      if ((CurrControl.type == WINDOW) &&
          (GSstricmp(CurrControl.gname, "top") == 0) &&
          (GSstricmp(CurrControl.control.name, "options") == 0))
      {
        GSmouseoff();
        GSdisplayhelpmessage(78); /* Press Run another simulation */
        GSmouseon();
        NEXT(Stage);
      }
      break;
    case PRESSRUNANOTHER :
      if ((CurrControl.type == MENU) &&
          (GSstricmp(CurrControl.gname, "options") == 0) &&
          (CurrControl.control.number == 1))
      {
        GSmouseoff();
        GSdisablebutton("top", "Pyrgraph", NO_DRAW);
        GSdisablebutton("toplifeexp", "setlife", DRAW);
        GSdisablebutton("topmig", "setmig", DRAW);
        GSdisablebutton("toptotalfert", "setfert", DRAW);
        GSdisablebutton("top", "simulate", DRAW);
        GSdisablebutton("top", "options", DRAW);
        GSdisablebutton("setfert", "incfert", NO_DRAW);
        GSdisablebutton("setfert", "incfyear", NO_DRAW);
        GSdisablebutton("setfert", "decfyear", NO_DRAW);
        GSdisablebutton("setfert", "okay", NO_DRAW);
        GSenablebutton("message", "wtcont", NO_DRAW);
        GSdisplayhelpmessage(79); /* Notice new simulation */
        GSmouseon();
        NEXT(Stage);
      }
      break;
    case CONTINUE7 :
      if ((CurrControl.type == WINDOW) &&
          (GSstricmp(CurrControl.gname, "message") == 0 ) &&
          (GSstricmp(CurrControl.control.name, "wtcont") == 0))
      {
        GSmouseoff();
        GSdisablebutton("message", "wtcont", NO_DRAW);
        GSenablebutton("toptotalfert", "setfert", DRAW);
        GSdisplayhelpmessage(60); /* Press Total Fert. */
        GSmouseon();
        GSclearlastcontrol();
        NEXT(Stage);
      }
      break;
    case PRESSTOTFERT :
      if ((CurrControl.type == WINDOW) &&
          (GSstricmp(CurrControl.gname, "toptotalfert") == 0) &&
          (GSstricmp(CurrControl.control.name, "setfert") == 0))
      {
        GSmouseoff();
        GSdisablebutton("setfert", "incfert", DRAW);
        GSdisplayhelpmessage(61); /* Press - under Children */
        GSmouseon();
        Drag_Used = FALSE;
        GSclearlastcontrol();
        NEXT(Stage);
      }
      break;
    case PRESSDECFERT :
      if ((CurrControl.type == WINDOW) &&
          (GSstricmp(CurrControl.gname, "setfert") == 0) &&
          (GSstricmp(CurrControl.control.name, "decfert") == 0))
      {
        GSdisablebutton("setfert", "incfert", DRAW);
        if (Drag_Used)
        {
          GSmouseoff();
          GSdisplayhelpmessage(62); /* Press + to 2010 */
          GSenablebutton("setfert", "incfyear", DRAW);
          GSmouseon();
          Drag_Used = FALSE;
          NEXT(Stage);
        }
        GSclearlastcontrol();
      }
      break;
    case INCFYEAR2010 :
      if ((CurrControl.type == WINDOW) &&
          (GSstricmp(CurrControl.gname, "setfert") == 0) &&
          (GSstricmp(CurrControl.control.name, "incfyear") == 0))
      {
        GSdisablebutton("setfert", "decfyear", DRAW);
        if (Drag_Used)
        {
          GSmouseoff();
          GSdisplayhelpmessage(63); /* Press done */
          GSdisablebutton("setfert", "incfyear", DRAW);
          GSdisablebutton("setfert", "decfyear", DRAW);
          GSenablebutton("setfert", "okay", DRAW);
          GSmouseon();
          Drag_Used = FALSE;
          NEXT(Stage);
        }
        GSclearlastcontrol();
      }
      break;
    case PRESSFERTDONE :
      if ((CurrControl.type == WINDOW) &&
          (GSstricmp(CurrControl.gname, "setfert") == 0) &&
          (GSstricmp(CurrControl.control.name, "okay") == 0))
      {
        GSmouseoff();
        GSdisablebutton("toptotalfert", "setfert", DRAW);
        GSenablebutton("top", "simulate", DRAW);
        GSdisplayhelpmessage(64); /* Press SIM--> to 2050 again */
        GSmouseon();
        Drag_Used = FALSE;
        NEXT(Stage);
      }
      break;
    case GOTO2050AGAIN :
      if (Drag_Used)
      {
        Drag_Used = FALSE;
        GSmouseoff();
        GSdisplayhelpmessage(65); /* Notice lower curve */
        GSdisablebutton("top", "unsimulate", DRAW);
        GSdisablebutton("top", "options", DRAW);
        GSdisablemenuitem("options", "p0");
        GSdisablemenuitem("options", "p1");
        GSdisablemenuitem("options", "p2");
        GSdisablemenuitem("options", "p3");
        GSdisablemenuitem("options", "p4");
        GSdisablemenuitem("options", "p5");
        GSenablebutton("message", "wtcont", DRAW);
        GSmouseon();
        NEXT(Stage);
      }
      break;
    case CONTINUE8 :
      if ((CurrControl.type == WINDOW) &&
          (GSstricmp(CurrControl.gname, "message") == 0 ) &&
          (GSstricmp(CurrControl.control.name, "wtcont") == 0))
      {
        GSmouseoff();
        GSdisablebutton("message", "wtcont", NO_DRAW);
        GSdisplayhelpmessage(66); /* Press Options */
        GSenablebutton("top", "options", DRAW);
        GSdisablemenuitem("options", "p0");
        GSdisablemenuitem("options", "p1");
        GSdisablemenuitem("options", "p2");
        GSdisablemenuitem("options", "p3");
        GSdisablemenuitem("options", "p4");
        GSenablemenuitem("options", "p5");
        GSmouseon();
        GSclearlastcontrol();
        NEXT(Stage);
      }
      break;
    case PRESSOPTIONS3 :
      if ((CurrControl.type == WINDOW) &&
          (GSstricmp(CurrControl.gname, "top") == 0) &&
          (GSstricmp(CurrControl.control.name, "options") == 0))
      {
        GSmouseoff();
        GSdisplayhelpmessage(67); /* Press Reset simulation */
        GSmouseon();
        NEXT(Stage);
      }
      break;
    case PRESSRESETSIM :
      if ((CurrControl.type == MENU) &&
          (GSstricmp(CurrControl.gname, "options") == 0) &&
          (CurrControl.control.number == 5))
      {
        GSmouseoff();
        GSdisablebutton("top", "Pyrgraph", NO_DRAW);
        GSdisablebutton("toplifeexp", "setlife", DRAW);
        GSdisablebutton("topmig", "setmig", DRAW);
        GSdisablebutton("toptotalfert", "setfert", DRAW);
        GSdisablebutton("top", "simulate", DRAW);
        GSdisablebutton("setfert", "incfert", NO_DRAW);
        GSdisablebutton("setfert", "incfyear", NO_DRAW);
        GSdisablebutton("setfert", "decfyear", NO_DRAW);
        GSdisablebutton("setfert", "okay", NO_DRAW);
        GSdisplayhelpmessage(68); /* Notice new simulation */
        GSdisablemenuitem("options", "p5");
        GSenablemenuitem("options", "p3");
        GSmouseon();
        NEXT(Stage);
      }
      break;
    case PRESSOPTIONS4 :
      if ((CurrControl.type == WINDOW) &&
          (GSstricmp(CurrControl.gname, "top") == 0) &&
          (GSstricmp(CurrControl.control.name, "options") == 0))
      {
        GSmouseoff();
        GSdisplayhelpmessage(69); /* Press View birth/death buttons */
        GSmouseon();
        NEXT(Stage);
      }
      break;
    case PRESSVIEWBUTTONS :
      if ((CurrControl.type == MENU) &&
          (GSstricmp(CurrControl.gname, "options") == 0) &&
          (CurrControl.control.number == 3))
      {
        GSmouseoff();
        GSdisablebutton("toplifeexp", "setlife", DRAW);
        GSdisablebutton("topmig", "setmig", DRAW);
        GSdisablebutton("toptotalfert", "setfert", DRAW);
        GSdisablebutton("top", "simulate", DRAW);
        GSdisablebutton("top", "options", DRAW);
        GSdisablebutton("topinfant", "Infantm", DRAW);
        GSdisablebutton("topinfant", "Infantp", DRAW);
        for (i=1; i < 6; i++)
        {
          sprintf(tempstrng, "Birth%dm", i);
          GSdisablebutton("topbirth", tempstrng, DRAW);
          sprintf(tempstrng, "Birth%dp", i);
          GSdisablebutton("topbirth", tempstrng, DRAW);
        }
        for (i=0; i < 9; i++)
        {
          sprintf(tempstrng, "Death%dm", i);
          GSdisablebutton("topdeath", tempstrng, DRAW);
          sprintf(tempstrng, "Death%dp", i);
          GSdisablebutton("topdeath", tempstrng, DRAW);
        }
        GSenablebutton("message", "wtcont", NO_DRAW);
        GSdisplayhelpmessage(50); /* Notice + and - buttons */
        GSmouseon();
        NEXT(Stage);
      }
      break;
    case CONTINUE9 :
      if ((CurrControl.type == WINDOW) &&
          (GSstricmp(CurrControl.gname, "message") == 0 ) &&
          (GSstricmp(CurrControl.control.name, "wtcont") == 0))
      {
        GSmouseoff();
        GSenablebutton("topdeath", "Death7m", DRAW);
        GSdisablebutton("message", "wtcont", NO_DRAW);
        GSdisplayhelpmessage(51); /* Press - button */
        Drag_Used = FALSE;
        GSmouseon();
        NEXT(Stage);
      }
      break;
    case PRESSDEATHMINUS :
      if ((CurrControl.type == WINDOW) &&
          (GSstricmp(CurrControl.gname, "topdeath") == 0 ) &&
          (GSstricmp(CurrControl.control.name, "death7m") == 0))
      {
        if (Drag_Used)
        {
          GSmouseoff();
          GSdisablebutton("topdeath", "Death7m", DRAW);
          GSdisplayhelpmessage(52); /* You can drag cohorts, press continue */
          GSenablebutton("message", "wtcont", DRAW);
          GSmouseon();
          NEXT(Stage);
        }
      }
      break;
    case CONTINUE10 :
      if ((CurrControl.type == WINDOW) &&
          (GSstricmp(CurrControl.gname, "message") == 0 ) &&
          (GSstricmp(CurrControl.control.name, "wtcont") == 0))
      {
        GSmouseoff();
        GSdisablebutton("message", "wtcont", NO_DRAW);
        GSenablebutton("top", "Pyrgraph", NO_DRAW);
        GSdisplayhelpmessage(53); /* Drag a cohort */
        Drag_Used = FALSE;
        GSmouseon();
        NEXT(Stage);
      }
      break;
    case DRAGCOHORT :
      if ( Drag_Used &&
           ((CurrControl.type == WINDOW) &&
            (GSstricmp(CurrControl.gname, "top") == 0 ) &&
            (GSstricmp(CurrControl.control.name, "Pyrgraph") == 0)))
      {
        GSmouseoff();
        GSenablebutton("top", "simulate", DRAW);
        GSdisplayhelpmessage(54); /* Press Sim again */
        Drag_Used = FALSE;
        GSmouseon();
        NEXT(Stage);
      }
      else
      {
        GSclearlastcontrol();
        Drag_Used = FALSE;
      }
      break;
    case GOTO2050FINAL :
      if (Drag_Used &&
          ((CurrControl.type == WINDOW) &&
           (GSstricmp(CurrControl.gname, "top") == 0 ) &&
           (GSstricmp(CurrControl.control.name, "simulate") == 0)))
      {
        Drag_Used = FALSE;
        GSmouseoff();
        GSdisplayhelpmessage(55); /* Notice lower curve */
        GSdisablebutton("top", "unsimulate", DRAW);
        GSdisablebutton("top", "options", DRAW);
        GSdisablemenuitem("options", "p0");
        GSdisablemenuitem("options", "p1");
        GSdisablemenuitem("options", "p2");
        GSdisablemenuitem("options", "p3");
        GSdisablemenuitem("options", "p4");
        GSdisablemenuitem("options", "p5");
        GSenablebutton("message", "wtcont", DRAW);
        GSmouseon();
        NEXT(Stage);
      }
      else
      {
        GSclearlastcontrol();
        Drag_Used = FALSE;
      }
      break;
    case CONTINUE11 :
      if ((CurrControl.type == WINDOW) &&
          (GSstricmp(CurrControl.gname, "message") == 0 ) &&
          (GSstricmp(CurrControl.control.name, "wtcont") == 0))
      {
        GSdisablebutton("message", "wtcont", NO_DRAW);
        NEXT(Stage);
      }
      break;
    case QUITTUT : /* final stage.  Clean up */
      quitwalkthru();
      break;
  } /* switch(Stage) */

  return(TRUE);
}

/***********************************************************************
  stop the walkthrough tutorial
***********************************************************************/
boolean quitwalkthru(void)
{
  BigString filename;

  GSelog(0, "End of Walk-Through Tutorial\n");
  GSmouseoff();
  GSinvisiblebutton("root", "qtwalk", DRAW);
  Stage = WT_NONE;			/* reset stage to INTRO */
  GSdeactivatefunction("walkthru");
  if (MapSelect)
	mapcancel();

  /* restore previous simulations */
  sprintf(filename, "%s%s", Home, WALKTHRU_SAVE);
  loadnamedfile(filename);
  if ((remove(filename)==-1))
	GSelog(0, "Odd error: could not remove walk-through save file |%s|.\n",
		WALKTHRU_SAVE);
  GSenablemenuitem("help", "p4");
  GSenablemenuitem("options", 0);

  InWalkThru = FALSE;
  GSdisplayhelpmessage(WTSaveMessage);
  GSmouseon();

  return(TRUE);
}

